@@ -1,4 +1,5 @@
 /* utility program to make a hex dump of a binary file (such as a mdb file) */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -18,17 +19,17 @@
 
 int main(int argc, char **argv)
 {
-   long                 i=0;
-   int                  j;
+   unsigned long        i=0;
+   unsigned int         j;
    unsigned char        data[17];
    FILE                 *in;
-   int                  length;
+   size_t               length;
    int                  pg=0;
    char                 addr[10];
    int                  jet4 = 0;
    int			pg_size = 2048;
 
-   if (argc < 1) {
+   if (argc < 2) {
 	fprintf(stderr, "Usage: mdb-dump <filename> [<page number>]\n\n");
 	exit(1);
    }
@@ -51,15 +52,19 @@ int main(int argc, char **argv)
 	exit(1);
    }
    fseek(in,0x14,SEEK_SET);
-   fread(data,1,1,in);
-   if (data[0]==0x01) {
+   length = fread(data,1,1,in);
+   if (!length) {
+	fprintf(stderr, "fread failed at position 0x14\n");
+	exit(1);
+   }
+   if (data[0]) {
 	jet4 = 1;
 	pg_size = 4096;
    } 
    fseek(in,(pg*pg_size),SEEK_SET);
    i = 0;
-   while (length = fread(data,1,16,in)) {
-      sprintf(addr, "%06x", i);
+   while ((length = fread(data,1,16,in))) {
+      sprintf(addr, "%06lx", i);
       //if (!strcmp(&addr[3],"000") || (!jet4 && !strcmp(&addr[3],"800")) &&
 		      //pg) break;
       if (!strcmp(&addr[3],"000") || (!jet4 && !strcmp(&addr[3],"800"))) {
