@@ -11,10 +11,13 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
+
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-help.h>
 #include "gmdb.h"
 
 extern GtkWidget *app;
@@ -35,7 +38,19 @@ gmdb_prefs_get_maxrows()
 }
 
 /* callbacks */
-void
+static void
+gmdb_prefs_help_cb(GtkWidget *w, gpointer data)
+{
+	GError *error = NULL;
+
+	gnome_help_display("gmdb.xml", "gmdb-prefs", &error);
+	if (error != NULL) {
+		g_warning ("%s", error->message);
+		g_error_free (error);
+	}
+}
+
+static void
 gmdb_prefs_save_cb(GtkWidget *w, GladeXML *xml)
 {
 	GtkWidget *entry;
@@ -51,7 +66,7 @@ gmdb_prefs_save_cb(GtkWidget *w, GladeXML *xml)
 	if (win) gtk_widget_destroy(win);
 }
 
-void
+static void
 gmdb_prefs_cancel_cb(GtkWidget *w, GladeXML *xml)
 {
 	GtkWidget *win;
@@ -82,6 +97,10 @@ gmdb_prefs_new()
 	g_signal_connect (G_OBJECT (button), "clicked",
 		G_CALLBACK (gmdb_prefs_save_cb), prefswin_xml);
 
+	button = glade_xml_get_widget (prefswin_xml, "help_button");
+	g_signal_connect (G_OBJECT (button), "clicked",
+		G_CALLBACK (gmdb_prefs_help_cb), prefswin_xml);
+
 	str = gnome_config_get_string("/gmdb/prefs/maxrows");
 	if (!str || !strlen(str)) {
 		str = "1000";
@@ -89,4 +108,7 @@ gmdb_prefs_new()
 		gnome_config_sync();
 	}
 	gtk_entry_set_text(GTK_ENTRY(entry), str);
+
+	prefswin = glade_xml_get_widget (prefswin_xml, "prefs_dialog");
+	return prefswin;
 }
