@@ -11,33 +11,26 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
+
+#include <gtk/gtkmessagedialog.h>
+#include <libgnome/gnome-i18n.h>
 #include "gmdb.h"
 
 #if SQL
 
-typedef struct GMdbSQLWindow {
-	GtkWidget *window;
-	GtkWidget *textbox;
-	GtkWidget *combo;
-	GtkWidget *clist;
-	GtkWidget *scroll;
-	GtkWidget *ctree;
-	GtkCTreeNode *current_node;
-	GList *history;
-} GMdbSQLWindow;
-
 GList *sql_list;
 
-extern GtkWidget *app;
 extern MdbHandle *mdb;
 extern MdbSQL *sql;
 
-void gmdb_sql_tree_populate(MdbHandle *mdb, GladeXML *xml);
-void gmdb_sql_load_query(GladeXML *xml, gchar *file_path);
+static void gmdb_sql_tree_populate (MdbHandle*, GladeXML*);
+static void gmdb_sql_load_query (GladeXML*, gchar*);
+static void gmdb_sql_save_query (GladeXML*, gchar*);
+static void gmdb_sql_save_as_cb (GtkWidget*, GladeXML*);
 
 void
 gmdb_sql_close_all()
@@ -45,128 +38,169 @@ gmdb_sql_close_all()
 	GladeXML *xml;
 	GtkWidget *win;
 
-	while (xml = g_list_nth_data(sql_list, 0)) {
+	while ((xml = g_list_nth_data(sql_list, 0))) {
 		win = glade_xml_get_widget (xml, "sql_window");
 		sql_list = g_list_remove(sql_list, xml);
 		if (win) gtk_widget_destroy(win);
 	}
 }
 
+gchar* gmdb_export_get_filepath (GladeXML*);	/* from table_export.c */
+
 /* callbacks */
-void
+static void
 gmdb_sql_write_rslt_cb(GtkWidget *w, GladeXML *xml)
 {
+	/* We need to re-run the whole query because some information is not stored
+	 * in the TreeStore, such as column types.
+	 */
 	gchar *file_path;
 	GladeXML *sql_xml;
-	GtkWidget *filesel;
+	GtkWidget *filesel, *dlg;
 	FILE *outfile;
 	int i;
 	int need_headers = 0;
-	int need_quote = 0;
 	gchar delimiter[11];
-	gchar quotechar;
+	gchar quotechar[5];
+	gchar escape_char[5];
+	int bin_mode;
 	gchar lineterm[5];
-	gchar *str, *buf;
-	int rows=0, n_columns;
-	char msg[100];
-	GtkWidget *treeview;
-	GtkTreeViewColumn *col;
-	GList *glist;
-	GtkTreeStore *store;
-	GtkTreeIter iter;
-	GValue value = { 0, };
-	
-	filesel = glade_xml_get_widget (xml, "export_dialog");
-	sql_xml = g_object_get_data(G_OBJECT(filesel), "sql_xml");
-	printf("sql_xml %lu\n",sql_xml);
 
-	gmdb_export_get_delimiter(xml, delimiter, 10);
-	gmdb_export_get_lineterm(xml, lineterm, 5);
-	need_quote = gmdb_export_get_quote(xml);
-	quotechar = gmdb_export_get_quotechar(xml);
+	guint len;
+	gchar *buf;
+	GtkTextIter start, end;
+	GtkTextBuffer *txtbuffer;
+	GtkWidget *textview;
+	char **bound_values;
+	int *bound_lens; 
+	MdbSQLColumn *sqlcol;
+	long row;
+	char *value;
+	size_t length;
+	MdbTableDef *table;
+	MdbColumn *col = NULL;
+
+	gmdb_export_get_delimiter(xml, delimiter, sizeof(delimiter));
+	gmdb_export_get_lineterm(xml, lineterm, sizeof(lineterm));
+	gmdb_export_get_quotechar(xml, quotechar, sizeof(quotechar));
+	gmdb_export_get_escapechar(xml, escape_char, sizeof(escape_char));
+	bin_mode = gmdb_export_get_binmode(xml);
 	need_headers = gmdb_export_get_headers(xml);
 	file_path = gmdb_export_get_filepath(xml);
 
 	if ((outfile=fopen(file_path, "w"))==NULL) {
-		gnome_warning_dialog("Unable to Open File!");
+		dlg = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (w)),
+		    GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
+		    _("Unable to open file."));
+		gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
 		return;
 	}
 
-	treeview = glade_xml_get_widget (sql_xml, "sql_results");
-	glist = gtk_tree_view_get_columns(treeview);
-	i = 0;
-	if (need_headers)  {
-		while (col = g_list_nth_data(glist, i)) {
-			if (i>0) fprintf(outfile,delimiter);
-			gmdb_print_quote(outfile, need_quote, quotechar, delimiter, gtk_tree_view_column_get_title(col));
-			fprintf(outfile,"%s", gtk_tree_view_column_get_title(col));
-			gmdb_print_quote(outfile, need_quote, quotechar, delimiter, gtk_tree_view_column_get_title(col));
-			i++;
-		}
-		fprintf(outfile,lineterm);
-		g_list_free(glist);
+	/* Get SQL */
+	filesel = glade_xml_get_widget (xml, "export_dialog");
+	sql_xml = g_object_get_data(G_OBJECT(filesel), "sql_xml");
+	//printf("sql_xml %p\n",sql_xml);
+	textview = glade_xml_get_widget(sql_xml, "sql_textview");
+	txtbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
+	len = gtk_text_buffer_get_char_count(txtbuffer);
+	gtk_text_buffer_get_iter_at_offset (txtbuffer, &start, 0);
+	gtk_text_buffer_get_iter_at_offset (txtbuffer, &end, len);
+	buf = gtk_text_buffer_get_text(txtbuffer, &start, &end, FALSE);
+
+
+	/* ok now execute it */
+	mdb_sql_run_query(sql, buf);
+	if (mdb_sql_has_error(sql)) {
+		GtkWidget* dlg = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (w)),
+		    GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
+		    "%s", mdb_sql_last_error(sql));
+		gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
+		mdb_sql_reset(sql);
+		
+		fclose(outfile);
+		gtk_widget_destroy(filesel);
+		return;
 	}
 
-	store = (GtkTreeStore *) gtk_tree_view_get_model(treeview);
-	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-	rows=0;
-	g_value_init (&value, G_TYPE_STRING);
-	do {
-		rows++;
-		n_columns = gtk_tree_model_get_n_columns(store);
-		for (i=0; i < n_columns; i++) {
-			if (i>0) fprintf(outfile,delimiter);
-			gtk_tree_model_get_value(GTK_TREE_MODEL(store), 
-					&iter, i, &value);
-			str = (gchar *) g_value_get_string(&value);
-			gmdb_print_quote(outfile, need_quote, quotechar, delimiter, str);
-			fprintf(outfile,"%s", str);
-			gmdb_print_quote(outfile, need_quote, quotechar, delimiter, str);
-			g_value_unset(&value);
+	bound_values = (char **) g_malloc(sql->num_columns * sizeof(char *));
+	bound_lens = (int *) g_malloc(sql->num_columns * sizeof(int));
+
+	for (i=0; i<sql->num_columns; i++) {
+		/* bind columns */
+		bound_values[i] = (char *) g_malloc0(MDB_BIND_SIZE); 
+		mdb_sql_bind_column(sql, i+1, bound_values[i], &bound_lens[i]);
+
+		/* display column titles */
+		if (need_headers) {
+			if (i>0)
+				fputs(delimiter, outfile);
+			sqlcol = g_ptr_array_index(sql->columns,i);
+			gmdb_print_col(outfile, sqlcol->name, quotechar[0]!='\0', MDB_TEXT, 0, quotechar, escape_char, bin_mode);
 		}
-		fprintf(outfile,lineterm);
-	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter));
+	}
+
+	row = 0;
+	while (mdb_fetch_row(sql->cur_table)) {
+		row++;
+		for (i=0; i<sql->num_columns; i++) { 
+			if (i>0)
+				fputs(delimiter, outfile);
+
+			sqlcol = g_ptr_array_index(sql->columns, i);
+
+			/* Find col matching sqlcol */
+			table = sql->cur_table;
+			for (i=0; i<table->num_cols; i++) {
+				col = g_ptr_array_index(table->columns, i);
+				if (!strcasecmp(sqlcol->name, col->name))
+					break;
+			}
+			/* assert(i!=table->num_cols). Can't happen, already checked. */
+
+			/* Don't quote NULLs */
+			if (bound_lens[i] && sqlcol->bind_type != MDB_OLE) {
+				if (col->col_type == MDB_OLE) {
+					value = mdb_ole_read_full(mdb, col, &length);
+				} else {
+					value = bound_values[i];
+					length = bound_lens[i];
+				}
+				gmdb_print_col(outfile, value, quotechar[0]!='\0', col->col_type, length, quotechar, escape_char, bin_mode);
+				if (col->col_type == MDB_OLE)
+					free(value);
+			}
+		}
+		fputs(lineterm, outfile);
+	}
+
+	/* free the memory used to bind */
+	for (i=0; i<sql->num_columns; i++) {
+		g_free(bound_values[i]);
+	}
+
+	mdb_sql_reset(sql);
+	g_free(buf);
 
 	fclose(outfile);
-	gtk_widget_destroy(filesel);
-	sprintf(msg,"%d Rows exported successfully.\n", rows);
-	gnome_ok_dialog(msg);
-}
-void
-gmdb_sql_write_cb(GtkWidget *w, GladeXML *xml)
-{
-	gchar *file_path;
-	GladeXML *sql_xml;
-	GtkWidget *filesel;
-
-	filesel = glade_xml_get_widget (xml, "file_dialog");
-	file_path = (gchar *) gtk_file_selection_get_filename (GTK_FILE_SELECTION(filesel));
-	sql_xml = g_object_get_data(G_OBJECT(filesel), "sql_xml");
-	gmdb_sql_save_query(sql_xml, file_path);
 
+	dlg = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (w)),
+	    GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
+	    _("%ld rows successfully exported."), row);
+	gtk_dialog_run (GTK_DIALOG (dlg));
+	gtk_widget_destroy (dlg);
+	
 	gtk_widget_destroy(filesel);
 }
-void
-gmdb_sql_load_cb(GtkWidget *w, GladeXML *xml)
-{
-	gchar *file_path;
-	GladeXML *sql_xml;
-	GtkWidget *filesel;
 
-	filesel = glade_xml_get_widget (xml, "file_dialog");
-	file_path = (gchar *) gtk_file_selection_get_filename (GTK_FILE_SELECTION(filesel));
-	sql_xml = g_object_get_data(G_OBJECT(filesel), "sql_xml");
-	gmdb_sql_load_query(sql_xml, file_path);
 
-	gtk_widget_destroy(filesel);
-}
-void
+static void
 gmdb_sql_results_cb(GtkWidget *w, GladeXML *xml)
 {
 	GladeXML *dialog_xml;
-	GtkWidget *but, *label;
+	GtkWidget *but;
 	GtkWidget *filesel;
-	gchar *str;
 
 	/* load the interface */
 	dialog_xml = glade_xml_new(GMDB_GLADEDIR "gmdb-export.glade", NULL, NULL);
@@ -190,7 +224,7 @@ gmdb_sql_results_cb(GtkWidget *w, GladeXML *xml)
 
 	g_object_set_data(G_OBJECT(filesel), "sql_xml", xml);
 }
-void
+static void
 gmdb_sql_save_cb(GtkWidget *w, GladeXML *xml)
 {
 	GtkWidget *textview;
@@ -204,49 +238,50 @@ gmdb_sql_save_cb(GtkWidget *w, GladeXML *xml)
 	}
 	gmdb_sql_save_query(xml, str);
 }
-void
+static void
 gmdb_sql_save_as_cb(GtkWidget *w, GladeXML *xml)
 {
-	GladeXML *dialog_xml;
-	GtkWidget *but;
-	GtkWidget *filesel;
-	gchar *str;
+	GtkWindow *parent_window = (GtkWindow *) glade_xml_get_widget (xml, "gmdb");
+	GtkWidget *dialog = gtk_file_chooser_dialog_new ("Save Query As",
+					      parent_window,
+					      GTK_FILE_CHOOSER_ACTION_SAVE,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+					      NULL);
 
-	/* load the interface */
-	dialog_xml = glade_xml_new(GMDB_GLADEDIR "gmdb-sql-file.glade", NULL, NULL);
-	/* connect the signals in the interface */
-	glade_xml_signal_autoconnect(dialog_xml);
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+	  {
+	    char *filename;
 
-	filesel = glade_xml_get_widget (dialog_xml, "file_dialog");
-	gtk_window_set_title(GTK_WINDOW(filesel), "Save Query As");
+	    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+	    gmdb_sql_save_query(xml, filename);
+	  }
 
-	but = glade_xml_get_widget (dialog_xml, "ok_button");
-	g_signal_connect (G_OBJECT (but), "clicked",
-		G_CALLBACK (gmdb_sql_write_cb), dialog_xml);
-
-	g_object_set_data(G_OBJECT(filesel), "sql_xml", xml);
+	gtk_widget_destroy (dialog);
 }
-void
+static void
 gmdb_sql_open_cb(GtkWidget *w, GladeXML *xml)
 {
-	GladeXML *dialog_xml;
-	GtkWidget *but;
-	GtkWidget *filesel;
-	gchar *str;
+	GtkWindow *parent_window = (GtkWindow *) glade_xml_get_widget (xml, "gmdb");
+	GtkWidget *dialog = gtk_file_chooser_dialog_new ("Open SQL Query",
+					      parent_window,
+					      GTK_FILE_CHOOSER_ACTION_OPEN,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+					      NULL);
 
-	/* load the interface */
-	dialog_xml = glade_xml_new(GMDB_GLADEDIR "gmdb-sql-file.glade", NULL, NULL);
-	/* connect the signals in the interface */
-	glade_xml_signal_autoconnect(dialog_xml);
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+	  {
+	    char *filename;
 
-	but = glade_xml_get_widget (dialog_xml, "ok_button");
-	g_signal_connect (G_OBJECT (but), "clicked",
-		G_CALLBACK (gmdb_sql_load_cb), dialog_xml);
+	    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+	    gmdb_sql_load_query(xml, filename);
+	  }
+
+	gtk_widget_destroy (dialog);
 
-	filesel = glade_xml_get_widget (dialog_xml, "file_dialog");
-	g_object_set_data(G_OBJECT(filesel), "sql_xml", xml);
 }
-void
+static void
 gmdb_sql_copy_cb(GtkWidget *w, GladeXML *xml)
 {
 	GtkTextBuffer *txtbuffer;
@@ -258,7 +293,7 @@ gmdb_sql_copy_cb(GtkWidget *w, GladeXML *xml)
 	txtbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
 	gtk_text_buffer_copy_clipboard(txtbuffer, clipboard);
 }
-void
+static void
 gmdb_sql_cut_cb(GtkWidget *w, GladeXML *xml)
 {
 	GtkTextBuffer *txtbuffer;
@@ -270,7 +305,7 @@ gmdb_sql_cut_cb(GtkWidget *w, GladeXML *xml)
 	txtbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
 	gtk_text_buffer_cut_clipboard(txtbuffer, clipboard, TRUE);
 }
-void
+static void
 gmdb_sql_paste_cb(GtkWidget *w, GladeXML *xml)
 {
 	GtkTextBuffer *txtbuffer;
@@ -282,7 +317,7 @@ gmdb_sql_paste_cb(GtkWidget *w, GladeXML *xml)
 	txtbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
 	gtk_text_buffer_paste_clipboard(txtbuffer, clipboard, NULL, TRUE);
 }
-void
+static void
 gmdb_sql_close_cb(GtkWidget *w, GladeXML *xml)
 {
 	GtkWidget *win;
@@ -291,7 +326,7 @@ gmdb_sql_close_cb(GtkWidget *w, GladeXML *xml)
 	if (win) gtk_widget_destroy(win);
 }
 
-void 
+static void 
 gmdb_sql_dnd_dataget_cb(
     GtkWidget *w, GdkDragContext *dc,
     GtkSelectionData *selection_data, guint info, guint t,
@@ -318,9 +353,9 @@ GtkTreeIter iter2;
 		selection_data,
 		GDK_SELECTION_TYPE_STRING,
 		8,  /* 8 bits per character. */
-		tablename, strlen(tablename));
+		(guchar*)tablename, strlen(tablename));
 }
-void gmdb_sql_dnd_datareceived_cb(
+static void gmdb_sql_dnd_datareceived_cb(
         GtkWidget *w,
         GdkDragContext *dc,
         gint x, gint y,
@@ -328,13 +363,11 @@ void gmdb_sql_dnd_datareceived_cb(
         guint info, guint t,
         GladeXML *xml)
 {
-gchar *buf;
 GtkTextIter iter;
 GtkTextBuffer *txtbuffer;
 GtkWidget *textview;
 
 	textview = glade_xml_get_widget(xml, "sql_textview");
-	buf = selection_data->data;
 	txtbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
 	if (gtk_text_buffer_get_char_count(txtbuffer)==0) {
 		gtk_text_buffer_get_iter_at_offset (txtbuffer, &iter, 0);
@@ -343,62 +376,68 @@ GtkWidget *textview;
 	gtk_widget_grab_focus(GTK_WIDGET(textview));
 }
 
-void
-gmdb_sql_select_hist_cb(GtkList *list, GladeXML *xml)
+static void
+gmdb_sql_select_hist_cb(GtkComboBox *combobox, GladeXML *xml)
 {
 	gchar *buf;
 	GtkTextBuffer *txtbuffer;
-	GtkWidget *combo, *textview;
+	GtkWidget *textview;
 
-	combo = glade_xml_get_widget(xml, "sql_combo");
-	if (!combo) return;
-	buf = (gchar *) gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo)->entry));
+	buf = gtk_combo_box_get_active_text(combobox);
 	if (!buf) return;
 	textview = glade_xml_get_widget(xml, "sql_textview");
 	txtbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
 	gtk_text_buffer_set_text(txtbuffer, buf, strlen(buf));
 }
 
-void
+static void
 gmdb_sql_execute_cb(GtkWidget *w, GladeXML *xml)
 {
-guint len;
-gchar *buf;
-gchar *bound_data[256];
-int i;
-MdbSQLColumn *sqlcol;
-gchar *titles[256];
-GtkTextBuffer *txtbuffer;
-GtkTextIter start, end;
-GtkWidget *textview, *combo, *treeview, *store;
-GList *history;
-GType *gtypes;
-GtkTreeIter iter;
-GtkTreeViewColumn *column;
-long row, maxrow;
+
+	guint len;
+	gchar *buf;
+	gchar *bound_data[256];
+	int i;
+	MdbSQLColumn *sqlcol;
+	GtkTextBuffer *txtbuffer;
+	GtkTextIter start, end;
+	GtkWidget *textview, *combobox, *treeview;
+	GtkTreeModel *store;
+	/*GtkWidget *window;*/
+	GType *gtypes;
+	GtkTreeIter iter;
+	GtkTreeViewColumn *column;
+	long row, maxrow;
+	/* GdkCursor *watch, *pointer; */
+
+	/*  need to figure out how to clock during the treeview recalc/redraw
+	window = glade_xml_get_widget(xml, "sql_window");
+	watch = gdk_cursor_new(GDK_WATCH);
+	gdk_window_set_cursor(GTK_WIDGET(window)->window, watch);
+	gdk_cursor_unref(watch);
+	*/
 
 	/* stuff this query on the history */
 	textview = glade_xml_get_widget(xml, "sql_textview");
-	combo = glade_xml_get_widget(xml, "sql_combo");
+	combobox = glade_xml_get_widget(xml, "sql_combo");
 	txtbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
-        len = gtk_text_buffer_get_char_count(txtbuffer);
+	len = gtk_text_buffer_get_char_count(txtbuffer);
 	gtk_text_buffer_get_iter_at_offset (txtbuffer, &start, 0);
 	gtk_text_buffer_get_iter_at_offset (txtbuffer, &end, len);
 	buf = gtk_text_buffer_get_text(txtbuffer, &start, &end, FALSE);
 
 	/* add to the history */
-	history = g_object_get_data(G_OBJECT(combo),"hist_list");
-	history = g_list_prepend(history, g_strdup(buf));
-	g_object_set_data(G_OBJECT(combo), "hist_list", history);
-	gtk_combo_set_popdown_strings(GTK_COMBO(combo), history);
+	gtk_combo_box_prepend_text(GTK_COMBO_BOX(combobox), buf);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), 0);
 
 	/* ok now execute it */
-	g_input_ptr = buf;
-	/* begin unsafe */
-	_mdb_sql(sql);
-	if (yyparse()) {
-		/* end unsafe */
-		gnome_warning_dialog("Couldn't parse SQL");
+	mdb_sql_run_query(sql, buf);
+	if (mdb_sql_has_error(sql)) {
+		GtkWidget* dlg = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (w)),
+		    GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
+		    "%s", mdb_sql_last_error(sql));
+		gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
 		mdb_sql_reset(sql);
 		return;
 	}
@@ -409,26 +448,24 @@ long row, maxrow;
 	for (i=0;i<sql->num_columns;i++) 
 		gtypes[i]=G_TYPE_STRING;
 
-	store = (GtkTreeStore *) gtk_tree_view_get_model(treeview);
+	store = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
 	if (store) {
-		i=0;
-		while (column = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), i)) {
+		while ((column = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), 0))) {
 			gtk_tree_view_remove_column(GTK_TREE_VIEW(treeview), column);
 		}
-		gtk_widget_destroy(store);
+		g_object_unref(store);
 	}
-	store = (GtkWidget *) gtk_list_store_newv(sql->num_columns, gtypes);
+	store = (GtkTreeModel*)gtk_list_store_newv(sql->num_columns, gtypes);
 	g_free(gtypes);
 
-	gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(store));
+	gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), store);
 	
 	GtkCellRenderer *renderer;
 	renderer = gtk_cell_renderer_text_new(); 
 
 	for (i=0;i<sql->num_columns;i++) { 
-		bound_data[i] = (char *) malloc(MDB_BIND_SIZE); 
-		bound_data[i][0] = '\0';
-        	mdbsql_bind_column(sql, i+1, bound_data[i]);
+		bound_data[i] = (char *) g_malloc0(MDB_BIND_SIZE); 
+        	mdb_sql_bind_column(sql, i+1, bound_data[i], NULL);
 		sqlcol = g_ptr_array_index(sql->columns,i);
 		column = gtk_tree_view_column_new_with_attributes(sqlcol->name, renderer, "text", i, NULL); 
 		gtk_tree_view_append_column(GTK_TREE_VIEW (treeview), column); 
@@ -450,20 +487,25 @@ long row, maxrow;
 
 	/* free the memory used to bind */
 	for (i=0;i<sql->num_columns;i++) {
-		free(bound_data[i]);
+		g_free(bound_data[i]);
 	}
 
 	mdb_sql_reset(sql);
 	g_free(buf);
+
+	/*
+	pointer = gdk_cursor_new(GDK_LEFT_PTR);
+	gdk_window_set_cursor(GTK_WIDGET(window)->window, pointer);
+	gdk_cursor_unref(pointer);
+	*/
+	
 }
 
 void
-gmdb_sql_new_cb(GtkWidget *w, gpointer data)
-{
+gmdb_sql_new_cb (GtkWidget *w, gpointer data) {
 	GtkTargetEntry src;
-	GtkWidget *mi, *but, *combo;
+	GtkWidget *mi, *but, *combobox;
 	GladeXML *sqlwin_xml;
-	GValue value = {0, };
 
 	/* load the interface */
 	sqlwin_xml = glade_xml_new(GMDB_GLADEDIR "gmdb-sql.glade", NULL, NULL);
@@ -484,9 +526,9 @@ gmdb_sql_new_cb(GtkWidget *w, gpointer data)
 	g_signal_connect (G_OBJECT (mi), "activate",
 		G_CALLBACK (gmdb_sql_save_as_cb), sqlwin_xml);
 
-	but = glade_xml_get_widget (sqlwin_xml, "save_as_button");
-	g_signal_connect (G_OBJECT (but), "clicked",
-		G_CALLBACK (gmdb_sql_save_as_cb), sqlwin_xml);
+	//but = glade_xml_get_widget (sqlwin_xml, "save_as_button");
+	//g_signal_connect (G_OBJECT (but), "clicked",
+	//	G_CALLBACK (gmdb_sql_save_as_cb), sqlwin_xml);
 
 	mi = glade_xml_get_widget (sqlwin_xml, "results_menu");
 	g_signal_connect (G_OBJECT (mi), "activate",
@@ -528,8 +570,8 @@ gmdb_sql_new_cb(GtkWidget *w, gpointer data)
 	g_signal_connect (G_OBJECT (mi), "activate",
 		G_CALLBACK (gmdb_sql_execute_cb), sqlwin_xml);
 
-	combo = glade_xml_get_widget(sqlwin_xml, "sql_combo");
-	g_signal_connect (G_OBJECT(GTK_COMBO(combo)->list), "selection-changed",
+	combobox = glade_xml_get_widget(sqlwin_xml, "sql_combo");
+	g_signal_connect (G_OBJECT(GTK_COMBO_BOX(combobox)), "changed",
 		G_CALLBACK (gmdb_sql_select_hist_cb), sqlwin_xml);
 
 	but = glade_xml_get_widget (sqlwin_xml, "execute_button");
@@ -572,6 +614,7 @@ gmdb_sql_new_cb(GtkWidget *w, gpointer data)
 	gtk_signal_connect( GTK_OBJECT(textview), "drag_data_received",
 		GTK_SIGNAL_FUNC(gmdb_sql_dnd_datareceived_cb), sqlwin_xml);
 	
+	//GValue value = {0, };
 	//but =  glade_xml_get_widget(sqlwin_xml, "results_button");
 	//g_value_init(&value, G_TYPE_STRING);
 	//g_value_set_static_string(&value, GMDB_ICONDIR "stock_export.png");
@@ -582,7 +625,7 @@ gmdb_sql_new_cb(GtkWidget *w, gpointer data)
 }
 
 /* functions */
-gchar *
+static gchar *
 gmdb_sql_get_basename(char *file_path)
 {
 	int i, len;
@@ -601,7 +644,7 @@ gmdb_sql_get_basename(char *file_path)
 	return basename;
 }
 
-void
+static void
 gmdb_sql_set_file(GladeXML *xml, gchar *file_name)
 {
 	GtkWidget *window, *textview;
@@ -619,9 +662,9 @@ gmdb_sql_set_file(GladeXML *xml, gchar *file_name)
 	textview = glade_xml_get_widget(xml, "sql_textview");
 	g_object_set_data(G_OBJECT(textview), "file_name", file_name);
 }
-void
-gmdb_sql_save_query(GladeXML *xml, gchar *file_path)
-{
+
+static void
+gmdb_sql_save_query (GladeXML *xml, gchar *file_path) {
 	FILE *out;
 	GtkWidget *textview;
         GtkTextBuffer *txtbuffer;
@@ -629,7 +672,11 @@ gmdb_sql_save_query(GladeXML *xml, gchar *file_path)
 	gchar *buf;
 
 	if (!(out=fopen(file_path, "w"))) {
-		gnome_warning_dialog("Unable to open file.");
+		GtkWidget* dlg = gtk_message_dialog_new (NULL,
+		    GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
+		    _("Unable to open file."));
+		gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
 		return;
 	}
 	textview = glade_xml_get_widget(xml, "sql_textview");
@@ -641,7 +688,7 @@ gmdb_sql_save_query(GladeXML *xml, gchar *file_path)
 	fclose(out);
 	gmdb_sql_set_file(xml, file_path);
 }
-void
+static void
 gmdb_sql_load_query(GladeXML *xml, gchar *file_path)
 {
 	FILE *in;
@@ -649,10 +696,13 @@ gmdb_sql_load_query(GladeXML *xml, gchar *file_path)
 	GtkWidget *textview;
         GtkTextBuffer *txtbuffer;
 	GtkTextIter start, end;
-	int len;
 
 	if (!(in=fopen(file_path, "r"))) {
-		gnome_warning_dialog("Unable to open file.");
+		GtkWidget* dlg = gtk_message_dialog_new (NULL,
+		    GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
+		    _("Unable to open file."));
+		gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
 		return;
 	}
 	textview = glade_xml_get_widget(xml, "sql_textview");
@@ -660,14 +710,14 @@ gmdb_sql_load_query(GladeXML *xml, gchar *file_path)
 	gtk_text_buffer_get_start_iter(txtbuffer, &start);
 	gtk_text_buffer_get_end_iter(txtbuffer, &end);
 	gtk_text_buffer_delete(txtbuffer, &start, &end);
-	while (len = fgets(buf, 255, in)) {
+	while (fgets(buf, 255, in) && (*buf != '\0')) {
 		gtk_text_buffer_get_end_iter(txtbuffer, &end);
-		gtk_text_buffer_insert(txtbuffer, &end, buf, len);
+		gtk_text_buffer_insert(txtbuffer, &end, buf, strlen(buf));
 	}
 	fclose(in);
 	gmdb_sql_set_file(xml, file_path);
 }
-void 
+static void 
 gmdb_sql_tree_populate(MdbHandle *mdb, GladeXML *xml)
 {
 int   i;
@@ -677,28 +727,25 @@ GtkTreeIter *iter2;
 	GtkWidget *tree = glade_xml_get_widget(xml, "sql_treeview");
 	GtkTreeStore *store = (GtkTreeStore *) gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
 
-	/* loop over each entry in the catalog */
+	/* add all user tables in catalog to tab */
 	for (i=0; i < mdb->num_catalog; i++) {
 		entry = g_ptr_array_index (mdb->catalog, i);
-
- 		/* if it's a table */
-		if (entry->object_type == MDB_TABLE) {
-			/* skip the MSys tables */
-			if (strncmp (entry->object_name, "MSys", 4)) {
-				/* add table to tab */
-				iter2 = g_malloc(sizeof(GtkTreeIter));
-				gtk_tree_store_append(store, iter2, NULL);
-				gtk_tree_store_set(store, iter2, 0, entry->object_name, -1);
-	     		}
-		} /* if MDB_TABLE */
+		if (mdb_is_user_table(entry)) {
+			iter2 = g_malloc(sizeof(GtkTreeIter));
+			gtk_tree_store_append(store, iter2, NULL);
+			gtk_tree_store_set(store, iter2, 0, entry->object_name, -1);
+		}
 	} /* for */
 }
 #else
 
 void
-gmdb_sql_new_cb(GtkWidget *w, gpointer data)
-{
-	gnome_ok_dialog("SQL support was not built in.\nRun configure with the --enable-sql option.");
+gmdb_sql_new_cb (GtkWidget *w, gpointer data) {
+	GtkWidget* dlg = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (w)),
+	    GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
+	    _("SQL support was not built.\nMake sure flex and yacc are installed at build time."));
+	gtk_dialog_run (GTK_DIALOG (dlg));
+	gtk_widget_destroy (dlg);
 }
 
 #endif
