@@ -12,9 +12,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "mdbtools.h"
@@ -65,136 +64,165 @@ char idx_to_text[] = {
 GPtrArray *
 mdb_read_indices(MdbTableDef *table)
 {
-MdbCatalogEntry *entry = table->entry;
-MdbHandle *mdb = entry->mdb;
-MdbFormatConstants *fmt = mdb->fmt;
-MdbIndex idx, *pidx;
-int i, j;
-int idx_num, key_num, col_num;
-int cur_pos;
-int name_sz, idx2_sz;
-gchar *tmpbuf;
+	MdbCatalogEntry *entry = table->entry;
+	MdbHandle *mdb = entry->mdb;
+	MdbFormatConstants *fmt = mdb->fmt;
+	MdbIndex *pidx;
+	unsigned int i, j, k;
+	int key_num, col_num, cleaned_col_num;
+	int cur_pos, name_sz, idx2_sz, type_offset;
+	int index_start_pg = mdb->cur_pg;
+	gchar *tmpbuf;
 
-/* FIX ME -- doesn't handle multipage table headers */
+	table->indices = g_ptr_array_new();
 
-        table->indices = g_ptr_array_new();
-
-        if (IS_JET4(mdb)) {
-		cur_pos = table->index_start + 52 * table->num_real_idxs;
-		idx2_sz = 27;
-		
-	} else {
+	if (IS_JET3(mdb)) {
 		cur_pos = table->index_start + 39 * table->num_real_idxs;
-		idx2_sz = 19;
+		idx2_sz = 20;
+		type_offset = 19;
+	} else {
+		cur_pos = table->index_start + 52 * table->num_real_idxs;
+		idx2_sz = 28;
+		type_offset = 23;
 	}
 
+	//fprintf(stderr, "num_idxs:%d num_real_idxs:%d\n", table->num_idxs, table->num_real_idxs);
+	/* num_real_idxs should be the number of indexes of type 2.
+	 * It's not always the case. Happens on Northwind Orders table.
+	 */
+	table->num_real_idxs = 0;
+	tmpbuf = (gchar *) g_malloc(idx2_sz);
 	for (i=0;i<table->num_idxs;i++) {
-		memset(&idx, '\0', sizeof(MdbIndex));
-		idx.table = table;
-		cur_pos +=4;
-		idx.index_num = read_pg_if_16(mdb, &cur_pos);
-		read_pg_if(mdb, &cur_pos, idx2_sz - 4);
-		cur_pos += idx2_sz - 4;
-		idx.index_type = mdb->pg_buf[cur_pos++]; 
-		mdb_append_index(table->indices, &idx);
+		read_pg_if_n(mdb, tmpbuf, &cur_pos, idx2_sz);
+		pidx = (MdbIndex *) g_malloc0(sizeof(MdbIndex));
+		pidx->table = table;
+		pidx->index_num = mdb_get_int16(tmpbuf, 4);
+		pidx->index_type = tmpbuf[type_offset];
+		g_ptr_array_add(table->indices, pidx);
+		/*
+		{
+			gint32 dumy0 = mdb_get_int32(tmpbuf, 0);
+			gint8 dumy1 = tmpbuf[8];
+			gint32 dumy2 = mdb_get_int32(tmpbuf, 9);
+			gint32 dumy3 = mdb_get_int32(tmpbuf, 13);
+			gint16 dumy4 = mdb_get_int16(tmpbuf, 17);
+			fprintf(stderr, "idx #%d: num2:%d type:%d\n", i, pidx->index_num, pidx->index_type);
+			fprintf(stderr, "idx #%d: %d %d %d %d %d\n", i, dumy0, dumy1, dumy2, dumy3, dumy4);
+		}*/
+		if (pidx->index_type!=2)
+			table->num_real_idxs++;
 	}
+	//fprintf(stderr, "num_idxs:%d num_real_idxs:%d\n", table->num_idxs, table->num_real_idxs);
+	g_free(tmpbuf);
 
 	for (i=0;i<table->num_idxs;i++) {
 		pidx = g_ptr_array_index (table->indices, i);
-		read_pg_if(mdb, &cur_pos, 0);
-		if (IS_JET4(mdb)) {
-			name_sz=read_pg_if_16(mdb, &cur_pos);
-			cur_pos += 2;
-			tmpbuf = g_malloc((name_sz + 1)*2);
-			read_pg_if_n(mdb, tmpbuf, &cur_pos, name_sz*2);
-			mdb_unicode2ascii(mdb, tmpbuf, 0, name_sz, pidx->name); 
-			g_free(tmpbuf);
-			cur_pos += name_sz;
+		if (IS_JET3(mdb)) {
+			name_sz=read_pg_if_8(mdb, &cur_pos);
 		} else {
-			name_sz=mdb->pg_buf[cur_pos++];
-			read_pg_if_n(mdb, pidx->name, &cur_pos, name_sz);
-			pidx->name[name_sz]='\0';		
-			cur_pos += name_sz;
+			name_sz=read_pg_if_16(mdb, &cur_pos);
 		}
-		//fprintf(stderr, "index name %s\n", pidx->name);
+		tmpbuf = g_malloc(name_sz);
+		read_pg_if_n(mdb, tmpbuf, &cur_pos, name_sz);
+		mdb_unicode2ascii(mdb, tmpbuf, name_sz, pidx->name, MDB_MAX_OBJ_NAME); 
+		g_free(tmpbuf);
+		//fprintf(stderr, "index %d type %d name %s\n", pidx->index_num, pidx->index_type, pidx->name);
 	}
 
-	cur_pos = table->index_start;
 	mdb_read_alt_pg(mdb, entry->table_pg);
-	mdb_read_pg(mdb, entry->table_pg);
-	idx_num=0;
+	mdb_read_pg(mdb, index_start_pg);
+	cur_pos = table->index_start;
 	for (i=0;i<table->num_real_idxs;i++) {
-		if (IS_JET4(mdb)) cur_pos += 4;
-		do {
-			pidx = g_ptr_array_index (table->indices, idx_num++);
-		} while (pidx && pidx->index_type==2);
-
-		/* if there are more real indexes than index entries left after
-		   removing type 2's decrement real indexes and continue.  Happens
-		   on Northwind Orders table.
-		*/
-		if (!pidx) {
-			table->num_real_idxs--;
+		if (!IS_JET3(mdb)) cur_pos += 4;
+		/* look for index number i */
+		for (j=0; j<table->num_idxs; ++j) {
+			pidx = g_ptr_array_index (table->indices, j);
+			if (pidx->index_type!=2 && pidx->index_num==i)
+				break;
+		}
+		if (j==table->num_idxs) {
+			fprintf(stderr, "ERROR: can't find index #%d.\n", i);
 			continue;
 		}
+		//fprintf(stderr, "index %d #%d (%s) index_type:%d\n", i, pidx->index_num, pidx->name, pidx->index_type);
 
 		pidx->num_rows = mdb_get_int32(mdb->alt_pg_buf, 
 				fmt->tab_cols_start_offset +
-				(i*fmt->tab_ridx_entry_size));
+				(pidx->index_num*fmt->tab_ridx_entry_size));
+		/*
+		fprintf(stderr, "ridx block1 i:%d data1:0x%08x data2:0x%08x\n",
+			i,
+			mdb_get_int32(mdb->pg_buf,
+				fmt->tab_cols_start_offset + pidx->index_num * fmt->tab_ridx_entry_size),
+			mdb_get_int32(mdb->pg_buf,
+				fmt->tab_cols_start_offset + pidx->index_num * fmt->tab_ridx_entry_size +4));
+		fprintf(stderr, "pidx->num_rows:%d\n", pidx->num_rows);*/
 
 		key_num=0;
 		for (j=0;j<MDB_MAX_IDX_COLS;j++) {
 			col_num=read_pg_if_16(mdb,&cur_pos);
-			cur_pos += 2;
-			if (col_num != 0xFFFF) {
-				/* set column number to a 1 based column number and store */
-				pidx->key_col_num[key_num]=col_num + 1;
-				if (mdb->pg_buf[cur_pos]) {
-					pidx->key_col_order[key_num]=MDB_ASC;
-				} else {
-					pidx->key_col_order[key_num]=MDB_DESC;
-				}
-				key_num++;
+			if (col_num == 0xFFFF) {
+				cur_pos++;
+				continue;
 			}
-			cur_pos++;
+			/* here we have the internal column number that does not
+			 * always match the table columns because of deletions */
+			cleaned_col_num = -1;
+			for (k=0; k<table->num_cols; k++) {
+				MdbColumn *col = g_ptr_array_index(table->columns,k);
+				if (col->col_num == col_num) {
+					cleaned_col_num = k;
+					break;
+				}
+			}
+			if (cleaned_col_num==-1) {
+				fprintf(stderr, "CRITICAL: can't find column with internal id %d in index %s\n",
+					col_num, pidx->name);
+				cur_pos++;
+				continue;
+			}
+			/* set column number to a 1 based column number and store */
+			pidx->key_col_num[key_num] = cleaned_col_num + 1;
+			pidx->key_col_order[key_num] =
+				(read_pg_if_8(mdb, &cur_pos)) ? MDB_ASC : MDB_DESC;
+			//fprintf(stderr, "component %d using column #%d (internally %d)\n", j, cleaned_col_num,  col_num);
+			key_num++;
 		}
 		pidx->num_keys = key_num;
+
 		cur_pos += 4;
+		//fprintf(stderr, "pidx->unknown_pre_first_pg:0x%08x\n", read_pg_if_32(mdb, &cur_pos));
 		pidx->first_pg = read_pg_if_32(mdb, &cur_pos);
-		cur_pos += 4;
-		read_pg_if(mdb, &cur_pos, 1);
-		pidx->flags = mdb->pg_buf[cur_pos++];
-		if (IS_JET4(mdb)) cur_pos += 9;
+		pidx->flags = read_pg_if_8(mdb, &cur_pos);
+		//fprintf(stderr, "pidx->first_pg:%d pidx->flags:0x%02x\n",	pidx->first_pg, pidx->flags);
+		if (!IS_JET3(mdb)) cur_pos += 9;
 	}
 	return NULL;
 }
 void
-mdb_index_hash_text(guchar *text, guchar *hash)
+mdb_index_hash_text(char *text, char *hash)
 {
-	int k;
+	unsigned int k;
 
 	for (k=0;k<strlen(text);k++) {
-		hash[k] = idx_to_text[text[k]];
+		int c = ((unsigned char *)(text))[k];
+		hash[k] = idx_to_text[c];
 		if (!(hash[k])) fprintf(stderr, 
-				"No translation available for %02x %d\n", 
-				text[k],text[k]);
+				"No translation available for %02x %d\n", c, c);
 	}
-	hash[strlen(text)]=0;
+	hash[strlen(text)]='\0';
 }
-guint32
-mdb_index_swap_int32(guint32 l)
+/*
+ * reverse the order of the column for hashing
+ */
+void
+mdb_index_swap_n(unsigned char *src, int sz, unsigned char *dest)
 {
-	unsigned char *c, *c2;
-	guint32 l2;
+	int i, j = 0;
 
-	c = (unsigned char *) &l;
-	c2 = (unsigned char *) &l2;
-	c2[0]=c[3];
-	c2[1]=c[2];
-	c2[2]=c[1];
-	c2[3]=c[0];
-
-	return l2;
+	for (i = sz-1; i >= 0; i--) {
+		dest[j++] = src[i];
+	}
 }
 void 
 mdb_index_cache_sarg(MdbColumn *col, MdbSarg *sarg, MdbSarg *idx_sarg)
@@ -208,7 +236,7 @@ mdb_index_cache_sarg(MdbColumn *col, MdbSarg *sarg, MdbSarg *idx_sarg)
 		break;
 
 		case MDB_LONGINT:
-		idx_sarg->value.i = mdb_index_swap_int32(sarg->value.i);
+		idx_sarg->value.i = GUINT32_SWAP_LE_BE(sarg->value.i);
 		//cache_int = sarg->value.i * -1;
 		c = (unsigned char *) &(idx_sarg->value.i);
 		c[0] |= 0x80;
@@ -252,9 +280,9 @@ int lastchar;
 }
 #endif
 int
-mdb_index_test_sargs(MdbHandle *mdb, MdbIndex *idx, unsigned char *buf, int len)
+mdb_index_test_sargs(MdbHandle *mdb, MdbIndex *idx, char *buf, int len)
 {
-	int i, j;
+	unsigned int i, j;
 	MdbColumn *col;
 	MdbTableDef *table = idx->table;
 	MdbSarg *idx_sarg;
@@ -321,15 +349,16 @@ mdb_index_pack_bitmap(MdbHandle *mdb, MdbIndexPage *ipg)
 {
 	int mask_bit = 0;
 	int mask_pos = 0x16;
-	int mask_byte;
+	int mask_byte = 0;
 	int elem = 0;
 	int len, start, i;
 
 	start = ipg->idx_starts[elem++];
 
 	while (start) {
+		//fprintf(stdout, "elem %d is %d\n", elem, ipg->idx_starts[elem]);
 		len = ipg->idx_starts[elem] - start;
-		fprintf(stdout, "len is %d\n", len);
+		//fprintf(stdout, "len is %d\n", len);
 		for (i=0; i < len; i++) {
 			mask_bit++;
 			if (mask_bit==8) {
@@ -340,7 +369,7 @@ mdb_index_pack_bitmap(MdbHandle *mdb, MdbIndexPage *ipg)
 			/* upon reaching the len, set the bit */
 		}
 		mask_byte = (1 << mask_bit) | mask_byte;
-		fprintf(stdout, "mask byte is %02x at %d\n", mask_byte, mask_pos);
+		//fprintf(stdout, "mask byte is %02x at %d\n", mask_byte, mask_pos);
 		start = ipg->idx_starts[elem++];
 	}
 	/* flush the last byte if any */
@@ -461,7 +490,7 @@ mdb_find_next_leaf(MdbHandle *mdb, MdbIndex *idx, MdbIndexChain *chain)
 			//printf("find_next_on_page returned 0\n");
 			return 0;
 		}
-		pg = mdb_pg_get_int24_msb(mdb, ipg->offset + ipg->len - 3);
+		pg = mdb_get_int32_msb(mdb->pg_buf, ipg->offset + ipg->len - 3) >> 8;
 		//printf("Looking at pg %lu at %lu %d\n", pg, ipg->offset, ipg->len);
 		ipg->offset += ipg->len;
 
@@ -571,6 +600,8 @@ mdb_index_find_next(MdbHandle *mdb, MdbIndex *idx, MdbIndexChain *chain, guint32
 	int passed = 0;
 	int idx_sz;
 	int idx_start = 0;
+	MdbColumn *col;
+	guint32 pg_row;
 
 	ipg = mdb_index_read_bottom_pg(mdb, idx, chain);
 
@@ -592,7 +623,8 @@ mdb_index_find_next(MdbHandle *mdb, MdbIndex *idx, MdbIndexChain *chain, guint32
 
 				if (!chain->last_leaf_found) return 0;
 				mdb_read_pg(mdb, chain->last_leaf_found);
-				chain->last_leaf_found = mdb_pg_get_int24(mdb, 0x0c);
+				chain->last_leaf_found = mdb_get_int32(
+					mdb->pg_buf, 0x0c);
 				//printf("next leaf %lu\n", chain->last_leaf_found);
 				mdb_read_pg(mdb, chain->last_leaf_found);
 				/* reuse the chain for cleanup mode */
@@ -605,28 +637,31 @@ mdb_index_find_next(MdbHandle *mdb, MdbIndex *idx, MdbIndexChain *chain, guint32
 					return 0;
 			}
 		}
-		*row = mdb->pg_buf[ipg->offset + ipg->len - 1];
-		*pg = mdb_pg_get_int24_msb(mdb, ipg->offset + ipg->len - 4);
+		pg_row = mdb_get_int32_msb(mdb->pg_buf, ipg->offset + ipg->len - 4);
+		*row = pg_row & 0xff;
+		*pg = pg_row >> 8;
 		//printf("row = %d pg = %lu ipg->pg = %lu offset = %lu len = %d\n", *row, *pg, ipg->pg, ipg->offset, ipg->len);
-		idx_sz = 4;
-		if (ipg->len - 4 < idx_sz) {
+		col=g_ptr_array_index(idx->table->columns,idx->key_col_num[0]-1);
+		idx_sz = mdb_col_fixed_size(col);
+		/* handle compressed indexes, single key indexes only? */
+		if (idx->num_keys==1 && idx_sz>0 && ipg->len - 4 < idx_sz) {
 			//printf("short index found\n");
-			//buffer_dump(ipg->cache_value, 0, idx_sz);
+			//mdb_buffer_dump(ipg->cache_value, 0, idx_sz);
 			memcpy(&ipg->cache_value[idx_sz - (ipg->len - 4)], &mdb->pg_buf[ipg->offset], ipg->len);
-			//buffer_dump(ipg->cache_value, 0, idx_sz);
+			//mdb_buffer_dump(ipg->cache_value, 0, idx_sz);
 		} else {
 			idx_start = ipg->offset + (ipg->len - 4 - idx_sz);
 			memcpy(ipg->cache_value, &mdb->pg_buf[idx_start], idx_sz);
 		}
 
 		//idx_start = ipg->offset + (ipg->len - 4 - idx_sz);
-		passed = mdb_index_test_sargs(mdb, idx, ipg->cache_value, idx_sz);
+		passed = mdb_index_test_sargs(mdb, idx, (char *)(ipg->cache_value), idx_sz);
 
 		ipg->offset += ipg->len;
 	} while (!passed);
 
 	//fprintf(stdout,"len = %d pos %d\n", ipg->len, ipg->mask_pos);
-	//buffer_dump(mdb->pg_buf, ipg->offset, ipg->offset+ipg->len-1);
+	//mdb_buffer_dump(mdb->pg_buf, ipg->offset, ipg->len);
 
 	return ipg->len;
 }
@@ -642,8 +677,8 @@ mdb_index_find_row(MdbHandle *mdb, MdbIndex *idx, MdbIndexChain *chain, guint32
 {
 	MdbIndexPage *ipg;
 	int passed = 0;
-	guint32 datapg;
-	guint16 datarow;
+	guint32 pg_row = (pg << 8) | (row & 0xff);
+	guint32 datapg_row;
 
 	ipg = mdb_index_read_bottom_pg(mdb, idx, chain);
 
@@ -671,10 +706,8 @@ mdb_index_find_row(MdbHandle *mdb, MdbIndex *idx, MdbIndexChain *chain, guint32
 				return 0;
 		}
 		/* test row and pg */
-		datarow = mdb->pg_buf[ipg->offset + ipg->len - 1];
-		datapg = mdb_pg_get_int24_msb(mdb, ipg->offset + ipg->len - 4);
-
-		if (datapg == pg && datarow == row) {
+		datapg_row = mdb_get_int32_msb(mdb->pg_buf, ipg->offset + ipg->len - 4);
+		if (pg_row == datapg_row) {
 			passed = 1;
 		}
 		ipg->offset += ipg->len;
@@ -686,11 +719,12 @@ mdb_index_find_row(MdbHandle *mdb, MdbIndex *idx, MdbIndexChain *chain, guint32
 
 void mdb_index_walk(MdbTableDef *table, MdbIndex *idx)
 {
-MdbHandle *mdb = table->entry->mdb;
-int cur_pos = 0;
-unsigned char marker;
-MdbColumn *col;
-int i;
+/*
+	MdbHandle *mdb = table->entry->mdb;
+	int cur_pos = 0;
+	unsigned char marker;
+	MdbColumn *col;
+	unsigned int i;
 
 	if (idx->num_keys!=1) return;
 
@@ -702,11 +736,12 @@ int i;
 		col=g_ptr_array_index(table->columns,idx->key_col_num[i]-1);
 		//printf("column %d coltype %d col_size %d (%d)\n",i,col->col_type, mdb_col_fixed_size(col), col->col_size);
 	}
+*/
 }
 void 
 mdb_index_dump(MdbTableDef *table, MdbIndex *idx)
 {
-	int i;
+	unsigned int i;
 	MdbColumn *col;
 
 	fprintf(stdout,"index number     %d\n", idx->index_num);
@@ -735,9 +770,9 @@ mdb_index_dump(MdbTableDef *table, MdbIndex *idx)
  */
 int mdb_index_compute_cost(MdbTableDef *table, MdbIndex *idx)
 {
-	int i;
+	unsigned int i;
 	MdbColumn *col;
-	MdbSarg *sarg;
+	MdbSarg *sarg = NULL;
 	int not_all_equal = 0;
 
 	if (!idx->num_keys) return 0;
@@ -831,7 +866,7 @@ int mdb_index_compute_cost(MdbTableDef *table, MdbIndex *idx)
 MdbStrategy 
 mdb_choose_index(MdbTableDef *table, int *choice)
 {
-	int i;
+	unsigned int i;
 	MdbIndex *idx;
 	int cost = 0;
 	int least = 99;
@@ -877,3 +912,13 @@ mdb_index_scan_free(MdbTableDef *table)
 		table->mdbidx = NULL;
 	}
 }
+
+void mdb_free_indices(GPtrArray *indices)
+{
+	unsigned int i;
+
+	if (!indices) return;
+	for (i=0; i<indices->len; i++)
+		g_free (g_ptr_array_index(indices, i));
+	g_ptr_array_free(indices, TRUE);
+}
