@@ -12,20 +12,17 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "mdbsql.h"
 #include <stdarg.h>
+#include "mdbsql.h"
 
 #ifdef DMALLOC
 #include "dmalloc.h"
 #endif
 
-void mdb_dump_results(MdbSQL *sql);
-
 #ifdef HAVE_WORDEXP_H
 #define HAVE_WORDEXP
 #include <wordexp.h>
@@ -33,16 +30,33 @@ void mdb_dump_results(MdbSQL *sql);
 
 char *g_input_ptr;
 
+/* Prevent warnings from -Wmissing-prototypes.  */
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int yyparse (void *YYPARSE_PARAM);
+#else
+int yyparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int yyparse (void);
+#else
+int yyparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
+
 void
-mdb_sql_error(char *fmt, ...)
+mdb_sql_error(MdbSQL* sql, char *fmt, ...)
 {
 va_list ap;
 
 	va_start(ap, fmt);
-	vfprintf (stderr,fmt, ap);
+	vsnprintf(sql->error_msg, sizeof(sql->error_msg), fmt, ap);
 	va_end(ap);
-	fprintf(stderr,"\n");
+
+	fprintf(stderr, "%s\n", sql->error_msg);
 }
+
 int mdb_sql_yyinput(char *buf, int need)
 {
 int cplen, have;
@@ -60,69 +74,86 @@ MdbSQL *mdb_sql_init()
 {
 MdbSQL *sql;
 
-	mdb_init();
 	sql = (MdbSQL *) g_malloc0(sizeof(MdbSQL));
 	sql->columns = g_ptr_array_new();
 	sql->tables = g_ptr_array_new();
 	sql->sarg_tree = NULL;
 	sql->sarg_stack = NULL;
 	sql->max_rows = -1;
-	
 
 	return sql;
 }
 
-int _parse(MdbSQL *sql, char *buf)
-{
-	g_input_ptr = buf;
+#ifndef _
+#define _(x) x
+#endif
+
+/**
+ * mdb_sql_run_query:
+ * @sql: MDB SQL object to execute the query on.
+ * @querystr: SQL query string to execute.
+ *
+ * Parses @querystr and executes it within the given @sql object.
+ *
+ * Returns: the updated MDB SQL object, or NULL on error
+ **/
+MdbSQL*
+mdb_sql_run_query (MdbSQL* sql, const gchar* querystr) {
+	g_return_val_if_fail (sql, NULL);
+	g_return_val_if_fail (querystr, NULL);
+
+	g_input_ptr = (gchar*) querystr;
+
+	/* calls to yyparse should be serialized for thread safety */
+
 	/* begin unsafe */
-	_mdb_sql(sql);
+	_mdb_sql (sql);
+	sql->error_msg[0]='\0';
 	if (yyparse()) {
 		/* end unsafe */
-		mdb_sql_reset(sql);
-		return 0;
-	} else {
-		return 1;
+		mdb_sql_error (sql, _("Could not parse '%s' command"), querystr);
+		mdb_sql_reset (sql);
+		return NULL;
 	}
-}
-int mdb_run_query(MdbSQL *sql, char *query)
-{
-	if (_parse(sql,query) && sql->cur_table) {
-		mdb_sql_bind_all(sql);
-		return 1;
-	} else {
-		return 0;
+
+	if (sql->cur_table == NULL) {
+		/* Invalid column name? (should get caught by mdb_sql_select,
+		 * but it appeared to happen anyway with 0.5) */
+		mdb_sql_error (sql, _("Got no result for '%s' command"), querystr);
+		return NULL;
 	}
+
+	mdb_sql_bind_all (sql);
+
+	return sql;
 }
+
 void mdb_sql_set_maxrow(MdbSQL *sql, int maxrow)
 {
 	sql->max_rows = maxrow;
 }
-void mdb_sql_free_column(MdbSQLColumn *c)
-{
-	if (c->name) g_free(c->name);
-	g_free(c);
-}
-MdbSQLColumn *mdb_sql_alloc_column()
-{
-MdbSQLColumn *c;
 
-	c = (MdbSQLColumn *) g_malloc(sizeof(MdbSQLColumn));
-	memset(c,0,sizeof(MdbSQLColumn));
-	return c;
-}
-void mdb_sql_free_table(MdbSQLTable *t)
+static void mdb_sql_free_columns(GPtrArray *columns)
 {
-	if (t->name) g_free(t->name);
-	g_free(t);
+	unsigned int i;
+	if (!columns) return;
+	for (i=0; i<columns->len; i++) {
+		MdbSQLColumn *c = (MdbSQLColumn *)g_ptr_array_index(columns, i);
+		g_free(c->name);
+		g_free(c);
+	}
+	g_ptr_array_free(columns, TRUE);
 }
-MdbSQLTable *mdb_sql_alloc_table()
+static void mdb_sql_free_tables(GPtrArray *tables)
 {
-MdbSQLTable *t;
-
-	t = (MdbSQLTable *) g_malloc(sizeof(MdbSQLTable));
-	memset(t,0,sizeof(MdbSQLTable));
-	return t;
+	unsigned int i;
+	if (!tables) return;
+	for (i=0; i<tables->len; i++) {
+		MdbSQLTable *t = (MdbSQLTable *)g_ptr_array_index(tables, i);
+		g_free(t->name);
+		g_free(t);
+	}
+	g_ptr_array_free(tables, TRUE);
 }
 
 void
@@ -130,20 +161,18 @@ mdb_sql_close(MdbSQL *sql)
 {
 	if (sql->mdb) {
 		mdb_close(sql->mdb);
-		mdb_free_handle(sql->mdb);
 		sql->mdb = NULL;
 	} else {
-		mdb_sql_error("Not connected.");
+		mdb_sql_error(sql, "Not connected.");
 	}
 }
 
 MdbHandle *mdb_sql_open(MdbSQL *sql, char *db_name)
 {
-int fail = 0;
-char *db_namep = db_name;
+	char *db_namep = db_name;
 
 #ifdef HAVE_WORDEXP
-wordexp_t words;
+	wordexp_t words;
 
 	if (wordexp(db_name, &words, 0)==0) {
 		if (words.we_wordc>0) 
@@ -152,21 +181,14 @@ wordexp_t words;
 	
 #endif
 
-	if (!(sql->mdb = mdb_open(db_namep))) {
-		if (!strstr(db_namep, ".mdb")) {
-			char *tmpstr = (char *) malloc(strlen(db_namep)+5);
-			strcpy(tmpstr,db_namep);
-			strcat(tmpstr,".mdb");
-			if (!(sql->mdb = mdb_open(tmpstr))) {
-				fail++;
-			}
-			free(tmpstr);
-		} else {
-			fail++;
-		}
+	sql->mdb = mdb_open(db_namep, MDB_NOFLAGS);
+	if ((!sql->mdb) && (!strstr(db_namep, ".mdb"))) {
+		char *tmpstr = (char *) g_strconcat(db_namep, ".mdb", NULL);
+		sql->mdb = mdb_open(tmpstr, MDB_NOFLAGS);
+		g_free(tmpstr);
 	}
-	if (fail) {
-		mdb_sql_error("Unable to locate database %s", db_name);
+	if (!sql->mdb) {
+		mdb_sql_error(sql, "Unable to locate database %s", db_name);
 	}
 
 #ifdef HAVE_WORDEXP
@@ -178,11 +200,7 @@ wordexp_t words;
 MdbSargNode *
 mdb_sql_alloc_node()
 {
-	MdbSargNode *node;
-
-	node = g_malloc0(sizeof(MdbSargNode));
-
-	return node;
+	return (MdbSargNode *) g_malloc0(sizeof(MdbSargNode));
 }
 void
 mdb_sql_free_tree(MdbSargNode *tree)
@@ -190,6 +208,7 @@ mdb_sql_free_tree(MdbSargNode *tree)
 
 	if (tree->left) mdb_sql_free_tree(tree->left);
 	if (tree->right) mdb_sql_free_tree(tree->right);
+	if (tree->parent) g_free(tree->parent);
 	g_free(tree);
 }
 void
@@ -229,7 +248,7 @@ mdb_sql_add_not(MdbSQL *sql)
 
 	left = mdb_sql_pop_node(sql);
 	if (!left) {
-		mdb_sql_error("parse error near 'NOT'");
+		mdb_sql_error(sql, "parse error near 'NOT'");
 		mdb_sql_reset(sql);
 		return;
 	}
@@ -246,7 +265,7 @@ mdb_sql_add_or(MdbSQL *sql)
 	left = mdb_sql_pop_node(sql);
 	right = mdb_sql_pop_node(sql);
 	if (!left || !right) {
-		mdb_sql_error("parse error near 'OR'");
+		mdb_sql_error(sql, "parse error near 'OR'");
 		mdb_sql_reset(sql);
 		return;
 	}
@@ -264,7 +283,7 @@ mdb_sql_add_and(MdbSQL *sql)
 	left = mdb_sql_pop_node(sql);
 	right = mdb_sql_pop_node(sql);
 	if (!left || !right) {
-		mdb_sql_error("parse error near 'AND'");
+		mdb_sql_error(sql, "parse error near 'AND'");
 		mdb_sql_reset(sql);
 		return;
 	}
@@ -346,13 +365,13 @@ mdb_sql_eval_expr(MdbSQL *sql, char *const1, int op, char *const2)
 			default: illop = 1;
 		}
 	} else {
-		mdb_sql_error("Comparison of strings and numbers not allowed.");
+		mdb_sql_error(sql, "Comparison of strings and numbers not allowed.");
 		/* the column and table names are no good now */
 		mdb_sql_reset(sql);
 		return 1;
 	}
 	if (illop) {
-		mdb_sql_error("Illegal operator used for comparision of literals.");
+		mdb_sql_error(sql, "Illegal operator used for comparision of literals.");
 		/* the column and table names are no good now */
 		mdb_sql_reset(sql);
 		return 1;
@@ -402,9 +421,9 @@ mdb_sql_all_columns(MdbSQL *sql)
 }
 int mdb_sql_add_column(MdbSQL *sql, char *column_name)
 {
-MdbSQLColumn *c;
+	MdbSQLColumn *c;
 
-	c = mdb_sql_alloc_column();
+	c = (MdbSQLColumn *) g_malloc0(sizeof(MdbSQLColumn));
 	c->name = g_strdup(column_name);
 	g_ptr_array_add(sql->columns, c);
 	sql->num_columns++;
@@ -412,9 +431,9 @@ MdbSQLColumn *c;
 }
 int mdb_sql_add_table(MdbSQL *sql, char *table_name)
 {
-MdbSQLTable *t;
+	MdbSQLTable *t;
 
-	t = mdb_sql_alloc_table();
+	t = (MdbSQLTable *) g_malloc0(sizeof(MdbSQLTable));
 	t->name = g_strdup(table_name);
 	t->alias = NULL;
 	g_ptr_array_add(sql->tables, t);
@@ -423,9 +442,9 @@ MdbSQLTable *t;
 }
 void mdb_sql_dump(MdbSQL *sql)
 {
-int i;
-MdbSQLColumn *c;
-MdbSQLTable *t;
+	unsigned int i;
+	MdbSQLColumn *c;
+	MdbSQLTable *t;
 
 	for (i=0;i<sql->num_columns;i++) {
 		c = g_ptr_array_index(sql->columns,i);
@@ -438,68 +457,41 @@ MdbSQLTable *t;
 }
 void mdb_sql_exit(MdbSQL *sql)
 {
-int i;
-MdbSQLColumn *c;
-MdbSQLTable *t;
-
-	for (i=0;i<sql->num_columns;i++) {
-		c = g_ptr_array_index(sql->columns,i);
-		if (c->name) g_free(c->name);
-	}
-	for (i=0;i<sql->num_tables;i++) {
-		t = g_ptr_array_index(sql->tables,i);
-		if (t->name) g_free(t->name);
-	}
-	if (sql->sarg_tree) {
-		mdb_sql_free_tree(sql->sarg_tree);
-		sql->sarg_tree = NULL;
-	}
-	g_list_free(sql->sarg_stack);
-	sql->sarg_stack = NULL;
-	g_ptr_array_free(sql->columns,TRUE);
-	g_ptr_array_free(sql->tables,TRUE);
-	if (sql->mdb) {
-		mdb_close(sql->mdb);	
-		mdb_free_handle(sql->mdb);	
-	}
+	mdb_sql_reset(sql); // Free memory
+	if (sql->mdb)
+		mdb_close(sql->mdb);
 }
 void mdb_sql_reset(MdbSQL *sql)
 {
-int i;
-MdbSQLColumn *c;
-MdbSQLTable *t;
-
 	if (sql->cur_table) {
 		mdb_index_scan_free(sql->cur_table);
+		if (sql->cur_table->sarg_tree) {
+			mdb_sql_free_tree(sql->cur_table->sarg_tree);
+			sql->cur_table->sarg_tree = NULL;
+		}
 		mdb_free_tabledef(sql->cur_table);
 		sql->cur_table = NULL;
 	}
-	if (sql->kludge_ttable_pg) {
-		g_free(sql->kludge_ttable_pg);
-		sql->kludge_ttable_pg = NULL;
-	}
-	for (i=0;i<sql->num_columns;i++) {
-		c = g_ptr_array_index(sql->columns,i);
-		mdb_sql_free_column(c);
-	}
-	for (i=0;i<sql->num_tables;i++) {
-		t = g_ptr_array_index(sql->tables,i);
-		mdb_sql_free_table(t);
-	}
+
+	/* Reset columns */
+	mdb_sql_free_columns(sql->columns);
+	sql->num_columns = 0;
+	sql->columns = g_ptr_array_new();
+
+	/* Reset tables */
+	mdb_sql_free_tables(sql->tables);
+	sql->num_tables = 0;
+	sql->tables = g_ptr_array_new();
+
+	/* Reset sargs */
 	if (sql->sarg_tree) {
 		mdb_sql_free_tree(sql->sarg_tree);
 		sql->sarg_tree = NULL;
 	}
 	g_list_free(sql->sarg_stack);
 	sql->sarg_stack = NULL;
-	g_ptr_array_free(sql->columns,TRUE);
-	g_ptr_array_free(sql->tables,TRUE);
 
 	sql->all_columns = 0;
-	sql->num_columns = 0;
-	sql->columns = g_ptr_array_new();
-	sql->num_tables = 0;
-	sql->tables = g_ptr_array_new();
 	sql->max_rows = -1;
 }
 static void print_break(int sz, int first)
@@ -529,99 +521,78 @@ int vlen;
 }
 void mdb_sql_listtables(MdbSQL *sql)
 {
-	int i;
+	unsigned int i;
 	MdbCatalogEntry *entry;
 	MdbHandle *mdb = sql->mdb;
-	MdbField fields[4];
-	unsigned char row_buffer[4096];
-	unsigned char *new_pg;
+	MdbField fields[1];
+	unsigned char row_buffer[MDB_PGSIZE];
 	int row_size;
-	MdbTableDef *ttable, *table = NULL;
-	MdbColumn *col, tcol;
-	MdbSQLColumn *sqlcol;
+	MdbTableDef *ttable;
+	gchar tmpstr[100];
+	int tmpsiz;
 
 	if (!mdb) {
-		mdb_sql_error("You must connect to a database first");
+		mdb_sql_error(sql, "You must connect to a database first");
 		return;
 	}
 	mdb_read_catalog (mdb, MDB_TABLE);
 
 	ttable = mdb_create_temp_table(mdb, "#listtables");
+	mdb_sql_add_temp_col(sql, ttable, 0, "Tables", MDB_TEXT, 30, 0);
 
-	memset(&tcol,0,sizeof(MdbColumn));
-	strcpy(tcol.name, "Tables");
-	tcol.col_size = 30;
-	tcol.col_type = MDB_TEXT;
-	tcol.is_fixed = 0;
-	mdb_temp_table_add_col(ttable, &tcol);
-	mdb_sql_add_column(sql, "Tables");
-	sqlcol = g_ptr_array_index(sql->columns,0);
-	sqlcol->disp_size = mdb_col_disp_size(&tcol);
-
-	/* blank out the pg_buf */
-	new_pg = mdb_new_data_pg(ttable->entry);
-	memcpy(mdb->pg_buf, new_pg, mdb->fmt->pg_size);
-	g_free(new_pg);
-
- 	/* loop over each entry in the catalog */
+ 	/* add all user tables in catalog to list */
  	for (i=0; i < mdb->num_catalog; i++) {
-     		entry = g_ptr_array_index (mdb->catalog, i);
-     		/* if it's a table */
-     		if (entry->object_type == MDB_TABLE) {
-       			if (strncmp (entry->object_name, "MSys", 4)) {
-          			//col = g_ptr_array_index(table->columns,0);
-   				fields[0].value = entry->object_name;
-   				fields[0].siz = strlen(entry->object_name); 
-   				fields[0].is_fixed = 0;
-   				fields[0].is_null = 0;
-   				fields[0].start = 0;
-   				fields[0].colnum = 0;
-
-				row_size = mdb_pack_row(ttable, row_buffer, 1, fields);
-				mdb_add_row_to_pg(ttable,row_buffer, row_size);
-				ttable->num_rows++;
-			}
+ 		entry = g_ptr_array_index (mdb->catalog, i);
+ 		if (mdb_is_user_table(entry)) {
+			//col = g_ptr_array_index(table->columns,0);
+			tmpsiz = mdb_ascii2unicode(mdb, entry->object_name, 0, tmpstr, 100);
+			mdb_fill_temp_field(&fields[0],tmpstr, tmpsiz, 0,0,0,0);
+			row_size = mdb_pack_row(ttable, row_buffer, 1, fields);
+			mdb_add_row_to_pg(ttable,row_buffer, row_size);
+			ttable->num_rows++;
 		}
 	}
-	sql->kludge_ttable_pg = g_memdup(mdb->pg_buf, mdb->fmt->pg_size);
 	sql->cur_table = ttable;
 
 }
+int
+mdb_sql_add_temp_col(MdbSQL *sql, MdbTableDef *ttable, int col_num, char *name, int col_type, int col_size, int is_fixed)
+{
+	MdbColumn tcol;
+	MdbSQLColumn *sqlcol;
+
+	mdb_fill_temp_col(&tcol, name, col_size, col_type, is_fixed);
+	mdb_temp_table_add_col(ttable, &tcol);
+	mdb_sql_add_column(sql, name);
+	sqlcol = g_ptr_array_index(sql->columns,col_num);
+	sqlcol->disp_size = mdb_col_disp_size(&tcol);
+
+	return 0;
+}
 void mdb_sql_describe_table(MdbSQL *sql)
 {
 	MdbTableDef *ttable, *table = NULL;
 	MdbSQLTable *sql_tab;
-	MdbCatalogEntry *entry;
 	MdbHandle *mdb = sql->mdb;
-	MdbColumn *col, tcol;
-	MdbSQLColumn *sqlcol;
-	int i;
-	char colsize[11];
-	MdbField fields[4];
+	MdbColumn *col;
+	unsigned int i;
+	MdbField fields[3];
 	char tmpstr[256];
-	unsigned char row_buffer[4096];
-	unsigned char *new_pg;
+	unsigned char row_buffer[MDB_PGSIZE];
 	int row_size;
+	gchar col_name[100], col_type[100], col_size[100];
+	int tmpsiz;
 
 	if (!mdb) {
-		mdb_sql_error("You must connect to a database first");
+		mdb_sql_error(sql, "You must connect to a database first");
 		return;
 	}
 
 	sql_tab = g_ptr_array_index(sql->tables,0);
 
-	mdb_read_catalog(mdb, MDB_TABLE);
-
-	for (i=0;i<mdb->num_catalog;i++) {
-		entry = g_ptr_array_index(mdb->catalog,i);
-		if (entry->object_type == MDB_TABLE &&
-			!strcasecmp(entry->object_name,sql_tab->name)) {
-				table = mdb_read_table(entry);
-				break;
-		}
-	}
+	table = mdb_read_table_by_name(mdb, sql_tab->name, MDB_TABLE);
 	if (!table) {
-		mdb_sql_error("%s is not a table in this database", sql_tab->name);
+		mdb_sql_error(sql, "%s is not a table in this database", sql_tab->name);
 		/* the column and table names are no good now */
 		mdb_sql_reset(sql);
 		return;
@@ -631,82 +602,38 @@ void mdb_sql_describe_table(MdbSQL *sql)
 
 	ttable = mdb_create_temp_table(mdb, "#describe");
 
-	memset(&tcol,0,sizeof(MdbColumn));
-	strcpy(tcol.name, "Column Name");
-	tcol.col_size = 30;
-	tcol.col_type = MDB_TEXT;
-	tcol.is_fixed = 0;
-	mdb_temp_table_add_col(ttable, &tcol);
-	mdb_sql_add_column(sql, "Column Name");
-	sqlcol = g_ptr_array_index(sql->columns,0);
-	sqlcol->disp_size = mdb_col_disp_size(&tcol);
+	mdb_sql_add_temp_col(sql, ttable, 0, "Column Name", MDB_TEXT, 30, 0);
+	mdb_sql_add_temp_col(sql, ttable, 1, "Type", MDB_TEXT, 20, 0);
+	mdb_sql_add_temp_col(sql, ttable, 2, "Size", MDB_TEXT, 10, 0);
 
-	memset(&tcol,0,sizeof(MdbColumn));
-	strcpy(tcol.name, "Type");
-	tcol.col_size = 20;
-	tcol.col_type = MDB_TEXT;
-	tcol.is_fixed = 0;
-	mdb_temp_table_add_col(ttable, &tcol);
-	mdb_sql_add_column(sql, "Type");
-	sqlcol = g_ptr_array_index(sql->columns,1);
-	sqlcol->disp_size = mdb_col_disp_size(&tcol);
+	for (i=0;i<table->num_cols;i++) {
 
-	memset(&tcol,0,sizeof(MdbColumn));
-	strcpy(tcol.name, "Size");
-	tcol.col_size = 10;
-	tcol.col_type = MDB_TEXT;
-	tcol.is_fixed = 0;
-	mdb_temp_table_add_col(ttable, &tcol);
-	mdb_sql_add_column(sql, "Size");
-	sqlcol = g_ptr_array_index(sql->columns,2);
-	sqlcol->disp_size = mdb_col_disp_size(&tcol);
+		col = g_ptr_array_index(table->columns,i);
+		tmpsiz = mdb_ascii2unicode(mdb, col->name, 0, col_name, 100);
+		mdb_fill_temp_field(&fields[0],col_name, tmpsiz, 0,0,0,0);
 
-	/* blank out the pg_buf */
-	new_pg = mdb_new_data_pg(ttable->entry);
-	memcpy(mdb->pg_buf, new_pg, mdb->fmt->pg_size);
-	g_free(new_pg);
+		strcpy(tmpstr, mdb_get_colbacktype_string(col));
+		tmpsiz = mdb_ascii2unicode(mdb, tmpstr, 0, col_type, 100);
+		mdb_fill_temp_field(&fields[1],col_type, tmpsiz, 0,0,0,1);
 
-     for (i=0;i<table->num_cols;i++) {
-
-          col = g_ptr_array_index(table->columns,i);
-   		fields[0].value = col->name; 
-   		fields[0].siz = strlen(col->name); 
-   		fields[0].is_fixed = 0;
-   		fields[0].is_null = 0;
-   		fields[0].start = 0;
-   		fields[0].colnum = 0;
-
-		strcpy(tmpstr, mdb_get_coltype_string(mdb->default_backend, col->col_type));
-   		fields[1].value = tmpstr; 
-   		fields[1].siz = strlen(tmpstr); 
-   		fields[1].is_fixed = 0;
-   		fields[1].is_null = 0;
-   		fields[1].start = 0;
-   		fields[1].colnum = 1;
-
-		sprintf(colsize,"%d",col->col_size);
-   		fields[2].value = colsize; 
-   		fields[2].siz = strlen(colsize); 
-   		fields[2].is_fixed = 0;
-   		fields[2].is_null = 0;
-   		fields[2].start = 0;
-   		fields[2].colnum = 2;
+		sprintf(tmpstr,"%d",col->col_size);
+		tmpsiz = mdb_ascii2unicode(mdb, tmpstr, 0, col_size, 100);
+		mdb_fill_temp_field(&fields[2],col_size, tmpsiz, 0,0,0,2);
 
 		row_size = mdb_pack_row(ttable, row_buffer, 3, fields);
 		mdb_add_row_to_pg(ttable,row_buffer, row_size);
 		ttable->num_rows++;
-     }
+	}
 
 	/* the column and table names are no good now */
 	//mdb_sql_reset(sql);
-	sql->kludge_ttable_pg = g_memdup(mdb->pg_buf, mdb->fmt->pg_size);
 	sql->cur_table = ttable;
 }
 
 int mdb_sql_find_sargcol(MdbSargNode *node, gpointer data)
 {
 	MdbTableDef *table = data;
-	int i;
+	unsigned int i;
 	MdbColumn *col;
 
 	if (!mdb_is_relational_op(node->op)) return 0;
@@ -724,8 +651,7 @@ int mdb_sql_find_sargcol(MdbSargNode *node, gpointer data)
 void 
 mdb_sql_select(MdbSQL *sql)
 {
-int i,j;
-MdbCatalogEntry *entry;
+unsigned int i,j;
 MdbHandle *mdb = sql->mdb;
 MdbTableDef *table = NULL;
 MdbSQLTable *sql_tab;
@@ -734,24 +660,15 @@ MdbSQLColumn *sqlcol;
 int found = 0;
 
 	if (!mdb) {
-		mdb_sql_error("You must connect to a database first");
+		mdb_sql_error(sql, "You must connect to a database first");
 		return;
 	}
 
 	sql_tab = g_ptr_array_index(sql->tables,0);
 
-	mdb_read_catalog(mdb, MDB_TABLE);
-
-	for (i=0;i<mdb->num_catalog;i++) {
-		entry = g_ptr_array_index(mdb->catalog,i);
-		if (entry->object_type == MDB_TABLE &&
-			!strcasecmp(entry->object_name,sql_tab->name)) {
-				table = mdb_read_table(entry);
-				break;
-		}
-	}
+	table = mdb_read_table_by_name(mdb, sql_tab->name, MDB_TABLE);
 	if (!table) {
-		mdb_sql_error("%s is not a table in this database", sql_tab->name);
+		mdb_sql_error(sql, "%s is not a table in this database", sql_tab->name);
 		/* the column and table names are no good now */
 		mdb_sql_reset(sql);
 		return;
@@ -762,11 +679,8 @@ int found = 0;
 
 	if (sql->all_columns) {
 		for (i=0;i<table->num_cols;i++) {
-			col=g_ptr_array_index(table->columns,i);
-			sqlcol = mdb_sql_alloc_column();
-			sqlcol->name = g_strdup(col->name);
-			g_ptr_array_add(sql->columns, sqlcol);
-			sql->num_columns++;
+			col = g_ptr_array_index(table->columns,i);
+			mdb_sql_add_column(sql, col->name);
 		}
 	}
 	/* verify all specified columns exist in this table */
@@ -782,7 +696,7 @@ int found = 0;
 			}
 		}
 		if (!found) {
-			mdb_sql_error("Column %s not found",sqlcol->name);
+			mdb_sql_error(sql, "Column %s not found",sqlcol->name);
 			mdb_sql_reset(sql);
 			return;
 		}
@@ -807,59 +721,39 @@ int found = 0;
 }
 
 void 
-mdb_sql_bind_column(MdbSQL *sql, int colnum, char *varaddr)
+mdb_sql_bind_column(MdbSQL *sql, int colnum, void *varaddr, int *len_ptr)
 {
-MdbTableDef *table = sql->cur_table;
-MdbSQLColumn *sqlcol;
-MdbColumn *col;
-int j;
+	MdbSQLColumn *sqlcol;
 
 	/* sql columns are traditionally 1 based, so decrement colnum */
 	sqlcol = g_ptr_array_index(sql->columns,colnum - 1);
-	for (j=0;j<table->num_cols;j++) {
-		col=g_ptr_array_index(table->columns,j);
-		if (!strcasecmp(sqlcol->name, col->name)) {
-			/* bind the column to its listed (SQL) position */
-			mdb_bind_column(table, j+1, varaddr);
-			break;
-		}
-	}
-}
-void 
-mdb_sql_bind_len(MdbSQL *sql, int colnum, int *len_ptr)
-{
-MdbTableDef *table = sql->cur_table;
-MdbSQLColumn *sqlcol;
-MdbColumn *col;
-int j;
-
-	/* sql columns are traditionally 1 based, so decrement colnum */
-	sqlcol = g_ptr_array_index(sql->columns,colnum - 1);
-	for (j=0;j<table->num_cols;j++) {
-		col=g_ptr_array_index(table->columns,j);
-		if (!strcasecmp(sqlcol->name, col->name)) {
-			/* bind the column length to its listed (SQL) position */
-			mdb_bind_len(table, j+1, len_ptr);
-			break;
-		}
-	}
+	mdb_bind_column_by_name(sql->cur_table, sqlcol->name, varaddr, len_ptr);
 }
 void 
 mdb_sql_bind_all(MdbSQL *sql)
 {
-int i;
+	unsigned int i;
 
 	for (i=0;i<sql->num_columns;i++) {
-		sql->bound_values[i] = (char *) malloc(MDB_BIND_SIZE);
-		sql->bound_values[i][0] = '\0';
-		mdb_sql_bind_column(sql, i+1, sql->bound_values[i]);
+		sql->bound_values[i] = g_malloc0(MDB_BIND_SIZE);
+		mdb_sql_bind_column(sql, i+1, sql->bound_values[i], NULL);
 	}
 }
+/*
+ * mdb_sql_fetch_row is now just a wrapper around mdb_fetch_row.
+ *   It is left here only for backward compatibility.
+ */
+int
+mdb_sql_fetch_row(MdbSQL *sql, MdbTableDef *table)
+{
+	return mdb_fetch_row(table);
+}
+
 void 
 mdb_sql_dump_results(MdbSQL *sql)
 {
-int j;
-MdbSQLColumn *sqlcol;
+	unsigned int j;
+	MdbSQLColumn *sqlcol;
 
 	/* print header */
 	for (j=0;j<sql->num_columns;j++) {
@@ -896,10 +790,9 @@ MdbSQLColumn *sqlcol;
 	fprintf(stdout,"\n");
 	/* clean up */
 	for (j=0;j<sql->num_columns;j++) {
-		if (sql->bound_values[j]) free(sql->bound_values[j]);
+		g_free(sql->bound_values[j]);
 	}
 
 	/* the column and table names are no good now */
 	mdb_sql_reset(sql);
 }
-
