@@ -11,9 +11,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /* this utility dumps the schema for an existing database */
@@ -43,7 +43,6 @@ int   i, j, k;
 MdbHandle *mdb;
 MdbCatalogEntry *entry;
 MdbTableDef *table;
-MdbColumn *col;
 char *tabname = NULL;
 int opt;
 
@@ -55,17 +54,14 @@ int opt;
   while ((opt=getopt(argc, argv, "T:"))!=-1) {
      switch (opt) {
        case 'T':
-         tabname = (char *) malloc(strlen(optarg)+1);
-         strcpy(tabname, optarg);
+         tabname = (char *) g_strdup(optarg);
          break;
      }
   }
  
- mdb_init();
-
  /* open the database */
 
- mdb = mdb_open (argv[optind]);
+ mdb = mdb_open (argv[optind], MDB_NOFLAGS);
 
  /* read the catalog */
  
@@ -80,10 +76,10 @@ int opt;
      /* if it's a table */
 
 	if (entry->object_type == MDB_TABLE) {
-		/* skip the MSys tables */
 		if ((tabname && !strcmp(entry->object_name,tabname)) ||
-			(!tabname )) {
-			 // && strncmp (entry->object_name, "MSys", 4))) {
+			(!tabname /* && mdb_is_user_table(entry) */)) {
+			int ret;
+
 	       		table = mdb_read_table(entry);
 
 	       		/* get the columns */
@@ -95,12 +91,15 @@ int opt;
 			fprintf(stdout,"Check 3: Checking for lost pages\n");
 			ret = dbcc_lost_pages(table);
 			//check_ret(table, ret);
+
+			mdb_free_tabledef(table);
 		}
 	}
+   }
 
-	mdb_free_handle (mdb);
-	mdb_exit();
+	g_free(tabname);
+	mdb_close (mdb);
 
-	exit(0);
+	return 0;
 }
 
