@@ -1,30 +1,25 @@
 /* MDB Tools - A library for reading MS Access database file
  * Copyright (C) 2000 Brian Bruns
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "mdbtools.h"
 
 #define MAX_ROW_SIZE 4096
 
-char *delimiter = ",";
-char header_rows = 0;
-
 void
 free_values(MdbField *fields, int num_fields)
 {
@@ -42,6 +37,7 @@ convert_field(MdbColumn *col, char *s, MdbField *field)
 	MdbAny any;
 
 	field->colnum = col->col_num;
+	field->is_fixed = col->is_fixed;
 	switch (col->col_type) {
 		case MDB_TEXT:
 			field->value = g_strdup(s);
@@ -59,14 +55,14 @@ convert_field(MdbColumn *col, char *s, MdbField *field)
 			if (*c) return 1;
 			field->siz = mdb_col_fixed_size(col);
 			field->value = g_malloc(field->siz);
-			_mdb_put_int16(field->value, 0, any.i);
+			mdb_put_int16(field->value, 0, any.i);
 			break;
 		case MDB_LONGINT:
 			any.i = strtol(s, &c, 16);
 			if (*c) return 1;
 			field->siz = mdb_col_fixed_size(col);
 			field->value = g_malloc(field->siz);
-			_mdb_put_int32(field->value, 0, any.i);
+			mdb_put_int32(field->value, 0, any.i);
 			break;
 		case MDB_BOOL:
 			if (*s == '1') {
@@ -82,13 +78,13 @@ convert_field(MdbColumn *col, char *s, MdbField *field)
 			any.d = strtod(s, &c);
 			if (*c) return 1;
 			field.value = g_malloc(mdb_col_fixed_size(col));
-			_mdb_put_single(field.value, 0, any.i);
+			mdb_put_single(field.value, 0, any.i);
 			break;
 		case MDB_DOUBLE:
 			any.d = strtod(s, &c);
 			if (*c) return 1;
 			field.value = g_malloc(mdb_col_fixed_size(col));
-			_mdb_put_double(field.value, 0, any.i);
+			mdb_put_double(field.value, 0, any.i);
 			break;
 		*/
 			/*
@@ -107,61 +103,62 @@ convert_field(MdbColumn *col, char *s, MdbField *field)
 	return 0;
 }
 int
-prep_row(MdbTableDef *table, unsigned char *line, MdbField *fields)
+prep_row(MdbTableDef *table, char *line, MdbField *fields, char *delim)
 {
 	MdbColumn *col;
-	char delims[20];
-	char *s;
-	int file_cols;
+	char **sarray, *s;
+	unsigned int i;
 
 	/*
 	 * pull apart the row and turn it into something respectable
 	 */
-	sprintf(delims, "%c\n", delimiter[0]);
-	s = strtok(line, delims);
-	file_cols = 0;
-	do {
-		if (file_cols >= table->num_cols) {
+	g_strdelimit(line, delim, '\n');
+	sarray = g_strsplit (line, "\n", 0);
+	for (i=0; (s = sarray[i]); i++) {
+		if (!strlen(s)) continue;
+		if (i >= table->num_cols) {
 			fprintf(stderr, "Number of columns in file exceeds number in table.\n");
+			g_strfreev(sarray);
 			return 0;
 		}
 		if (s[0]=='"' && s[strlen(s)-1]=='"') {
 			s[strlen(s)-1] = '\0';
-			s++;
+			memmove(s+1, s, strlen(s));
 		}
 		printf("field = %s\n", s);
-		col = g_ptr_array_index (table->columns, file_cols);
-		if (convert_field(col, s, &fields[file_cols])) {
-			fprintf(stderr, "Format error in column %d\n", file_cols+1);
+		col = g_ptr_array_index (table->columns, i);
+		if (convert_field(col, s, &fields[i])) {
+			fprintf(stderr, "Format error in column %d\n", i+1);
+			g_strfreev(sarray);
 			return 0;
 		}
-		file_cols++;
-	} while (s = strtok(NULL, delims));		
+	}
+	g_strfreev(sarray);
 
 	/*
 	 * verify number of columns in table against number in row
 	 */
-	if (file_cols < table->num_cols) {
-		fprintf(stderr, "Row has %d columns, but table has %d\n", file_cols, table->num_cols);
-		free_values(fields, file_cols);
+	if (i < table->num_cols) {
+		fprintf(stderr, "Row has %d columns, but table has %d\n", i, table->num_cols);
+		free_values(fields, i);
 		return 0;
 	}
-	return file_cols;
+	return i-1;
 }
 int
 main(int argc, char **argv)
 {
 	int i, row;
 	MdbHandle *mdb;
-	MdbCatalogEntry *entry;
 	MdbTableDef *table;
 	MdbField fields[256];
-	unsigned char line[MAX_ROW_SIZE];
+	char line[MAX_ROW_SIZE];
 	int num_fields;
 	/* doesn't handle tables > 256 columns.  Can that happen? */
 	int  opt;
-	char *s;
 	FILE *in;
+	char delimiter[2] = ",";
+	char header_rows = 0;
 
 	while ((opt=getopt(argc, argv, "H:d:"))!=-1) {
 		switch (opt) {
@@ -169,8 +166,7 @@ main(int argc, char **argv)
 			header_rows = atol(optarg);
 		break;
 		case 'd':
-			delimiter = (char *) malloc(strlen(optarg)+1);
-			strcpy(delimiter, optarg);
+			delimiter[0] = optarg[0];
 		break;
 		default:
 		break;
@@ -189,29 +185,19 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	mdb_init();
-
-	if (!(mdb = _mdb_open(argv[optind], TRUE))) {
+	if (!(mdb = mdb_open(argv[optind], MDB_WRITABLE))) {
 		exit(1);
 	}
 	
-	mdb_read_catalog(mdb, MDB_TABLE);
-
-	for (i=0;i<mdb->num_catalog;i++) {
-		entry = g_ptr_array_index(mdb->catalog,i);
-		if (entry->object_type == MDB_TABLE &&
-			!strcmp(entry->object_name,argv[argc-2])) {
-			table = mdb_read_table(entry);
-			mdb_read_columns(table);
-			mdb_read_indices(table);
-			mdb_rewind_table(table);
-			break;
-		}
-	}
+	table = mdb_read_table_by_name(mdb, argv[argc-2], MDB_TABLE);
 	if (!table) {
 		fprintf(stderr,"Table %s not found in database\n", argv[argc-2]);
 		exit(1);
 	}
+	mdb_read_columns(table);
+	mdb_read_indices(table);
+	mdb_rewind_table(table);
+
 	/*
 	 * open the CSV file and read any header rows
 	 */
@@ -221,11 +207,14 @@ main(int argc, char **argv)
 		exit(1);
 	}
 	for (i=0;i<header_rows;i++)
-		fgets(line, MAX_ROW_SIZE, in);
+		if (!fgets(line, MAX_ROW_SIZE, in)) {
+			fprintf(stderr, "Error while reading header column #%d. Check -H parameter.\n", i);
+			exit(1);
+		}
 
 	row = 1;
 	while (fgets(line, MAX_ROW_SIZE, in)) {
-		num_fields = prep_row(table, line, fields);
+		num_fields = prep_row(table, line, fields, delimiter);
 		if (!num_fields) {
 			fprintf(stderr, "Aborting import at row %d\n", row);
 			exit(1);
@@ -236,10 +225,9 @@ main(int argc, char **argv)
 		mdb_insert_row(table, num_fields, fields);
 	}
 
+	mdb_free_tabledef(table);
 	fclose(in);
-	mdb_free_handle(mdb);
-	mdb_exit();
-
-	exit(0);
+	mdb_close(mdb);
+	return 0;
 }
 
