@@ -1,27 +1,49 @@
 /* MDB Tools - A library for reading MS Access database file
  * Copyright (C) 2000 Brian Bruns
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
-#include <config.h>
 
 #include <stdio.h>
-#ifdef HAVE_READLINE
-#include <readline/readline.h>
-#endif
+
+#ifdef HAVE_LIBREADLINE
+#  if defined(HAVE_READLINE_READLINE_H)
+#    include <readline/readline.h>
+#  elif defined(HAVE_READLINE_H)
+#    include <readline.h>
+#  else
+/* no readline.h */
+extern char *readline ();
+#  endif
+char *cmdline = NULL;
+#endif /* HAVE_LIBREADLINE */
+
+#ifdef HAVE_READLINE_HISTORY
+#  if defined(HAVE_READLINE_HISTORY_H)
+#    include <readline/history.h>
+#  elif defined(HAVE_HISTORY_H)
+#    include <history.h>
+#  else
+/* no history.h */
+extern void add_history ();
+extern int write_history ();
+extern int read_history ();
+extern void clear_history ();
+#  endif
+#endif /* HAVE_READLINE_HISTORY */
+
 #include <string.h>
 #include "mdbsql.h"
 
@@ -29,28 +51,28 @@
 #include "dmalloc.h"
 #endif
 
-void dump_results(MdbSQL *sql);
-void dump_results_pp(MdbSQL *sql);
-int yyparse(void);
+void dump_results(FILE *out, MdbSQL *sql, char *delimiter);
+void dump_results_pp(FILE *out, MdbSQL *sql);
 
 #if SQL
 
 int headers = 1;
 int footers = 1;
 int pretty_print = 1;
-char *delimiter;
 int showplan = 0;
 int noexec = 0;
 
+#ifdef HAVE_READLINE_HISTORY
 #define HISTFILE ".mdbhistory"
+#endif
 
-#ifndef HAVE_READLINE
+#ifndef HAVE_LIBREADLINE
 char *readline(char *prompt)
 {
 char *buf, line[1000];
 int i = 0;
 
-	printf("%s",prompt);
+	puts(prompt);
 	if (! fgets(line,1000,stdin)) {
 		return NULL;
 	}
@@ -65,24 +87,15 @@ int i = 0;
 
 	return buf;
 }
-void add_history(char *s)
-{
-}
 #endif
 
-int parse(MdbSQL *sql, char *buf)
-{
-	g_input_ptr = buf;
-	/* begin unsafe */
-	_mdb_sql(sql);
-	if (yyparse()) {
-		/* end unsafe */
-		fprintf(stderr, "Couldn't parse SQL\n");
-		mdb_sql_reset(sql);
-		return 1;
-	} else {
-		return 0;
+static int strlen_utf(const char *s) {
+	int len = 0;
+	while (*s) {
+		if ((*s++ & 0xc0) != 0x80)
+			len++;
 	}
+	return len;
 }
 
 void
@@ -90,6 +103,10 @@ do_set_cmd(MdbSQL *sql, char *s)
 {
 	char *level1, *level2;
 	level1 = strtok(s, " \t\n");
+	if (!level1) {
+		printf("Usage: set [stats|showplan|noexec] [on|off]\n");
+		return;
+	}
 	if (!strcmp(level1,"stats")) {
 		level2 = strtok(NULL, " \t");
 		if (!level2) {
@@ -103,6 +120,7 @@ do_set_cmd(MdbSQL *sql, char *s)
 			mdb_dump_stats(sql->mdb);
 		} else {
 			printf("Unknown stats option %s\n", level2);
+			printf("Usage: set stats [on|off]\n");
 		}
 	} else if (!strcmp(level1,"showplan")) {
 		level2 = strtok(NULL, " \t");
@@ -116,6 +134,7 @@ do_set_cmd(MdbSQL *sql, char *s)
 			showplan=0;
 		} else {
 			printf("Unknown showplan option %s\n", level2);
+			printf("Usage: set showplan [on|off]\n");
 		}
 	} else if (!strcmp(level1,"noexec")) {
 		level2 = strtok(NULL, " \t");
@@ -128,20 +147,22 @@ do_set_cmd(MdbSQL *sql, char *s)
 		} else if (!strcmp(level2,"off")) {
 			noexec=0;
 		} else {
-			printf("Unknown showplan option %s\n", level2);
+			printf("Unknown noexec option %s\n", level2);
+			printf("Usage: set noexec [on|off]\n");
 		}
 	} else {
 		printf("Unknown set command %s\n", level1);
+		printf("Usage: set [stats|showplan|noexec] [on|off]\n");
 	}
 }
 
 int
-read_file(char *s, int line, int *bufsz, char *mybuf)
+read_file(char *s, int line, unsigned int *bufsz, char *mybuf)
 {
 	char *fname;
 	FILE *in;
 	char buf[256];
-	int cursz = 0;	
+	unsigned int cursz = 0;	
 	int lines = 0;	
 
 	fname = s;
@@ -159,8 +180,10 @@ read_file(char *s, int line, int *bufsz, char *mybuf)
 			mybuf = (char *) realloc(mybuf, *bufsz);
 		}	
 		strcat(mybuf, buf);
+#ifdef HAVE_READLINE_HISTORY
 		/* don't record blank lines */
 		if (strlen(buf)) add_history(buf);
+#endif
 		strcat(mybuf, "\n");
 		lines++;
 		printf("%d => %s",line+lines, buf);
@@ -168,11 +191,12 @@ read_file(char *s, int line, int *bufsz, char *mybuf)
 	return lines;
 }
 void 
-run_query(MdbSQL *sql, char *mybuf)
+run_query(FILE *out, MdbSQL *sql, char *mybuf, char *delimiter)
 {
 	MdbTableDef *table;
 
-	if (!parse(sql, mybuf) && sql->cur_table) {
+	mdb_sql_run_query(sql, mybuf);
+	if (!mdb_sql_has_error(sql)) {
 		if (showplan) {
 			table = sql->cur_table;
 			if (table->sarg_tree) mdb_sql_dump_node(table->sarg_tree, 0);
@@ -185,104 +209,88 @@ run_query(MdbSQL *sql, char *mybuf)
 			mdb_sql_reset(sql);
 			return;
 		}
-		mdb_sql_bind_all(sql);
 		if (pretty_print)
-			dump_results_pp(sql);
+			dump_results_pp(out, sql);
 		else
-			dump_results(sql);
+			dump_results(out, sql, delimiter);
 	}
 }
 
-void print_value(char *v, int sz, int first)
+void print_value(FILE *out, char *v, int sz, int first)
 {
 int i;
 int vlen;
 
-	if (first) {
-		fprintf(stdout,"|");
-	}
-	vlen = strlen(v);
-	for (i=0;i<sz;i++) {
-		fprintf(stdout,"%c",i >= vlen ? ' ' : v[i]);
-	}
-	fprintf(stdout,"|");
+	if (first)
+		fputc('|', out);
+	vlen = strlen_utf(v);
+	fputs(v, out);
+	for (i=vlen;i<sz;i++)
+		fputc(' ', out);
+	fputc('|', out);
 }
-static void print_break(int sz, int first)
+static void print_break(FILE *out, int sz, int first)
 {
 int i;
-	if (first) {
-		fprintf(stdout,"+");
-	}
-	for (i=0;i<sz;i++) {
-		fprintf(stdout,"-");
-	}
-	fprintf(stdout,"+");
+	if (first)
+		fputc('+', out);
+	for (i=0;i<sz;i++)
+		fputc('-', out);
+	fputc('+', out);
+}
+void print_rows_retrieved(FILE *out, unsigned long row_count)
+{
+	if (!row_count) 
+		fprintf(out, "No Rows retrieved\n");
+	else if (row_count==1)
+		fprintf(out, "1 Row retrieved\n");
+	else 
+		fprintf(out, "%lu Rows retrieved\n", row_count);
+	fflush(out);
 }
 void
-dump_results(MdbSQL *sql)
+dump_results(FILE *out, MdbSQL *sql, char *delimiter)
 {
-	int j;
+	unsigned int j;
 	MdbSQLColumn *sqlcol;
 	unsigned long row_count = 0;
-	int rows, rc, i;
-	MdbHandle *mdb = sql->mdb;
-	MdbFormatConstants *fmt = mdb->fmt;
 
 	if (headers) {
 		for (j=0;j<sql->num_columns-1;j++) {
 			sqlcol = g_ptr_array_index(sql->columns,j);
-			fprintf(stdout, "%s%s", sqlcol->name, delimiter);
+			fprintf(out, "%s%s", sqlcol->name,
+				delimiter ? delimiter : "\t");
 		}
 		sqlcol = g_ptr_array_index(sql->columns,sql->num_columns-1);
-		fprintf(stdout, "%s", sqlcol->name);
-		fprintf(stdout,"\n");
+		fprintf(out, "%s", sqlcol->name);
+		fprintf(out,"\n");
+		fflush(out);
 	}
-	if (sql->kludge_ttable_pg) {
-		memcpy(mdb->pg_buf, sql->kludge_ttable_pg, fmt->pg_size);
-		rows = mdb_pg_get_int16(mdb,fmt->row_count_offset);
-	for (i = 0; i < rows; i++) {
-		rc = mdb_read_row(sql->cur_table, i);
-		row_count++;
-  		for (j=0;j<sql->num_columns-1;j++) {
-			sqlcol = g_ptr_array_index(sql->columns,j);
-			fprintf(stdout, "%s%s", sql->bound_values[j], delimiter);
-		}
-		sqlcol = g_ptr_array_index(sql->columns,sql->num_columns-1);
-		fprintf(stdout, "%s", sql->bound_values[sql->num_columns-1]);
-		fprintf(stdout,"\n");
-	}
-	}else {
 	while(mdb_fetch_row(sql->cur_table)) {
 		row_count++;
   		for (j=0;j<sql->num_columns-1;j++) {
 			sqlcol = g_ptr_array_index(sql->columns,j);
-			fprintf(stdout, "%s%s", sql->bound_values[j], delimiter);
+			fprintf(out, "%s%s", (char*)(sql->bound_values[j]),
+				delimiter ? delimiter : "\t");
 		}
 		sqlcol = g_ptr_array_index(sql->columns,sql->num_columns-1);
-		fprintf(stdout, "%s", sql->bound_values[sql->num_columns-1]);
-		fprintf(stdout,"\n");
-	}
+		fprintf(out, "%s", (char*)(sql->bound_values[sql->num_columns-1]));
+		fprintf(out,"\n");
+		fflush(out);
 	}
 	if (footers) {
-		if (!row_count) 
-			fprintf(stdout, "No Rows retrieved\n");
-		else if (row_count==1)
-			fprintf(stdout, "1 Row retrieved\n");
-		else 
-			fprintf(stdout, "%lu Rows retrieved\n", row_count);
+		print_rows_retrieved(out, row_count);
 	}
+
 	mdb_sql_reset(sql);
 }
 
 void 
-dump_results_pp(MdbSQL *sql)
+dump_results_pp(FILE *out, MdbSQL *sql)
 {
-	int j;
+	unsigned int j;
 	MdbSQLColumn *sqlcol;
 	unsigned long row_count = 0;
-	int rows, rc, i;
-	MdbHandle *mdb = sql->mdb;
-	MdbFormatConstants *fmt = mdb->fmt;
 
 	/* print header */
 	if (headers) {
@@ -290,105 +298,87 @@ dump_results_pp(MdbSQL *sql)
 			sqlcol = g_ptr_array_index(sql->columns,j);
 			if (strlen(sqlcol->name)>sqlcol->disp_size)
 				sqlcol->disp_size = strlen(sqlcol->name);
-			print_break(sqlcol->disp_size, !j);
+			print_break(out, sqlcol->disp_size, !j);
 		}
-		fprintf(stdout,"\n");
+		fprintf(out,"\n");
+		fflush(out);
 		for (j=0;j<sql->num_columns;j++) {
 			sqlcol = g_ptr_array_index(sql->columns,j);
-			print_value(sqlcol->name,sqlcol->disp_size,!j);
+			print_value(out, sqlcol->name,sqlcol->disp_size,!j);
 		}
-		fprintf(stdout,"\n");
+		fprintf(out,"\n");
+		fflush(out);
 	}
 
 	for (j=0;j<sql->num_columns;j++) {
 		sqlcol = g_ptr_array_index(sql->columns,j);
-		print_break(sqlcol->disp_size, !j);
+		print_break(out, sqlcol->disp_size, !j);
 	}
-	fprintf(stdout,"\n");
+	fprintf(out,"\n");
+	fflush(out);
 
 	/* print each row */
-	if (sql->kludge_ttable_pg) {
-		memcpy(mdb->pg_buf, sql->kludge_ttable_pg, fmt->pg_size);
-		rows = mdb_pg_get_int16(mdb,fmt->row_count_offset);
-	for (i = 0; i < rows; i++) {
-		rc = mdb_read_row(sql->cur_table, i);
-		row_count++;
-  		for (j=0;j<sql->num_columns;j++) {
-			sqlcol = g_ptr_array_index(sql->columns,j);
-			print_value(sql->bound_values[j],sqlcol->disp_size,!j);
-		}
-		fprintf(stdout,"\n");
-	}
-	} else {
 	while(mdb_fetch_row(sql->cur_table)) {
 		row_count++;
   		for (j=0;j<sql->num_columns;j++) {
 			sqlcol = g_ptr_array_index(sql->columns,j);
-			print_value(sql->bound_values[j],sqlcol->disp_size,!j);
+			print_value(out, sql->bound_values[j],sqlcol->disp_size,!j);
 		}
-		fprintf(stdout,"\n");
-	}
+		fprintf(out,"\n");
+		fflush(out);
 	}
 
 	/* footer */
 	for (j=0;j<sql->num_columns;j++) {
 		sqlcol = g_ptr_array_index(sql->columns,j);
-		print_break(sqlcol->disp_size, !j);
+		print_break(out, sqlcol->disp_size, !j);
 	}
-	fprintf(stdout,"\n");
+	fprintf(out,"\n");
+	fflush(out);
 	if (footers) {
-		if (!row_count) 
-			fprintf(stdout, "No Rows retrieved\n");
-		else if (row_count==1)
-			fprintf(stdout, "1 Row retrieved\n");
-		else 
-			fprintf(stdout, "%lu Rows retrieved\n", row_count);
+		print_rows_retrieved(out, row_count);
 	}
 
 	/* clean up */
 	for (j=0;j<sql->num_columns;j++) {
-		if (sql->bound_values[j]) free(sql->bound_values[j]);
+		g_free(sql->bound_values[j]);
 	}
 
-	/* the column and table names are no good now */
 	mdb_sql_reset(sql);
 }
 
-void myexit(int r)
-{
-	free(delimiter);
-	exit(r);
-}
 int
 main(int argc, char **argv)
 {
-char *s;
+char *s = NULL;
 char prompt[20];
-int line = 1;
+int line = 0;
 char *mybuf;
-int bufsz = 4096;
-int done = 0;
+unsigned int bufsz;
 MdbSQL *sql;
 int opt;
 FILE *in = NULL, *out = NULL;
 char *home = getenv("HOME");
 char *histpath;
+char *delimiter = NULL;
 
 
+#ifdef HAVE_READLINE_HISTORY
 	if (home) {
-		histpath = (char *)malloc(strlen(home) + strlen(HISTFILE) + 2);
-		sprintf(histpath,"%s/%s",home,HISTFILE);
+		histpath = (char *) g_strconcat(home, "/", HISTFILE, NULL);
 		read_history(histpath);
-		free(histpath);
+		g_free(histpath);
 	}
+#endif
+	/* If input is coming from a pipe */
 	if (!isatty(fileno(stdin))) {
 		in = stdin;
 	}
+
 	while ((opt=getopt(argc, argv, "HFpd:i:o:"))!=-1) {
 		switch (opt) {
 		        case 'd':
-				delimiter = malloc(strlen(optarg)+1);
-				strcpy(delimiter, optarg);
+				delimiter = (char *) g_strdup(optarg);
 				break;
 		        case 'p':
 				pretty_print=0;
@@ -418,9 +408,6 @@ char *histpath;
 				exit(1);
 		}
 	}
-	if (!delimiter) {
-		delimiter = strdup("\t");
-	}
 	/* initialize the SQL engine */
 	sql = mdb_sql_init();
 	if (argc>optind) {
@@ -429,26 +416,40 @@ char *histpath;
 
 	/* give the buffer an initial size */
 	bufsz = 4096;
-	mybuf = (char *) malloc(bufsz);
+	mybuf = (char *) g_malloc(bufsz);
 	mybuf[0]='\0';
 
-	if (in) {
-		s=malloc(256);
-		if (!fgets(s, 256, in)) myexit(0);
-	} else {
-		sprintf(prompt,"1 => ");
-		s=readline(prompt);
-		if (!s) done = 1;
-		if (s && (!strcmp(s,"exit") || !strcmp(s,"quit") || !strcmp(s,"bye")))
-			done = 1;
-	}
-	while (!done) {
-		if (line==1 && !strncmp(s,"set ", 4)) {
-			do_set_cmd(sql, &s[4]);
+	while (1) {
+		line ++;
+		if (s) free(s);
+
+		if (in) {
+			s=malloc(256);
+			if ((!s) || (!fgets(s, 256, in))) {
+				/* if we have something in the buffer, run it */
+				if (strlen(mybuf))
+					run_query((out) ? out : stdout,
+					          sql, mybuf, delimiter);
+				break;
+			}
+			if (s[strlen(s)-1]=='\n')
+				s[strlen(s)-1]=0;
+		} else {
+			sprintf(prompt, "%d => ", line);
+			s=readline(prompt);
+			if (!s)
+				break;
+		}
+
+		if (!strcmp(s,"exit") || !strcmp(s,"quit") || !strcmp(s,"bye"))
+			break;
+
+		if (line==1 && (!strncmp(s,"set ",4) || !strcmp(s,"set"))) {
+			do_set_cmd(sql, &s[3]);
 			line = 0;
 		} else if (!strcmp(s,"go")) {
 			line = 0;
-			run_query(sql, mybuf);
+			run_query((out) ? out : stdout, sql, mybuf, delimiter);
 			mybuf[0]='\0';
 		} else if (!strcmp(s,"reset")) {
 			line = 0;
@@ -458,54 +459,41 @@ char *histpath;
 		} else {
 			while (strlen(mybuf) + strlen(s) > bufsz) {
 				bufsz *= 2;
-				mybuf = (char *) realloc(mybuf, bufsz);
+				mybuf = (char *) g_realloc(mybuf, bufsz);
 			}
+#ifdef HAVE_READLINE_HISTORY
 			/* don't record blank lines */
 			if (strlen(s)) add_history(s);
+#endif
 			strcat(mybuf,s);
 			/* preserve line numbering for the parser */
 			strcat(mybuf,"\n");
 		}
-		if (s) free(s);
-		if (in) {
-			s=malloc(256);
-			if (!fgets(s, 256, in)) {
-				/* if we have something in the buffer, run it */
-				if (strlen(mybuf)) 
-					run_query(sql, mybuf);
-				myexit(0);
-			}
-			if (s[strlen(s)-1]=='\n') s[strlen(s)-1]=0;
-		} else {
-			sprintf(prompt,"%d => ",++line);
-			s=readline(prompt);
-			if (!s) done = 1;
-		}
-		if( s && (!strcmp(s,"exit") || !strcmp(s,"quit") || !strcmp(s,"bye"))) {
-			done = 1;
-		}
 	}
 	mdb_sql_exit(sql);	
 
-	free(mybuf);
+	g_free(mybuf);
+	g_free(delimiter);
 	if (s) free(s);
+	if (out) fclose(out);
+	if ((in) && (in != stdin)) fclose(in);
 
+#ifdef HAVE_READLINE_HISTORY
 	if (home) {
-		histpath = (char *)malloc(strlen(home) + strlen(HISTFILE) + 2);
-		sprintf(histpath,"%s/%s",home,HISTFILE);
+		histpath = (char *) g_strconcat(home, "/", HISTFILE, NULL);
 		write_history(histpath);
-		free(histpath);
+		g_free(histpath);
+		clear_history();
 	}
+#endif
 
-	myexit(0);
-
-	return 0; /* make gcc -Wall happy */
+	return 0;
 }
 #else
 int main(int argc, char **argv)
 {
 	fprintf(stderr,"You must configure using --enable-sql to get SQL support\n");
 
-	exit(-1);
+	return -1;
 }
 #endif
