@@ -1,26 +1,25 @@
 /* MDB Tools - A library for reading MS Access database file
  * Copyright (C) 2000-2004 Brian Bruns
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
+
+#include <locale.h>
 #include "mdbtools.h"
 #include "mdbver.h"
 #include "mdbprivate.h"
-#include <locale.h>
 
 #ifdef DMALLOC
 #include "dmalloc.h"
@@ -60,23 +59,30 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	mdb_init();
-
-	if (!(mdb = mdb_open(argv[optind]))) {
+	if (!(mdb = mdb_open(argv[optind], MDB_NOFLAGS))) {
 		fprintf(stderr,_("Error: unable to open file %s\n"),argv[optind]);
 		exit(1);
 	}
-	if (IS_JET3(mdb)) {
+	switch(mdb->f->jet_version) {
+	case MDB_VER_JET3:
 		printf("JET3\n");
-	} else if (IS_JET4(mdb)) {
+		break;
+	case MDB_VER_JET4:
 		printf("JET4\n");
-	} else {
+		break;
+	case MDB_VER_ACCDB_2007:
+		printf("ACE12\n");
+		break;
+	case MDB_VER_ACCDB_2010:
+		printf("ACE14\n");
+		break;
+	default:
 		printf(_("unknown database version\n"));
+		break;
 	}
 	
-	mdb_free_handle(mdb);
-	mdb_exit();
+	mdb_close(mdb);
 
-	exit(0);
+	return 0;
 }
 
