@@ -11,9 +11,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "mdbtools.h"
@@ -24,20 +24,18 @@ main(int argc, char **argv)
 	MdbHandle *mdb;
 
 
-	if (argc<2) {
-		fprintf(stderr,"Usage: %s <file> [<objtype>]\n",argv[0]);
+	if (argc<2 || argc>3) {
+		fprintf(stderr, "Usage: %s <file> [<objtype>]\n", argv[0]);
+		fprintf(stderr, "Use objtype -1 for all\n");
 		exit(1);
 	}
 	
-	mdb_init();
-
-	mdb = mdb_open(argv[1]);
+	mdb = mdb_open(argv[1], MDB_NOFLAGS);
 
 	mdb_dump_catalog(mdb,(argc > 2) ? atoi(argv[2]) : MDB_TABLE); 
 
-	mdb_free_handle(mdb);
-	mdb_exit();
+	mdb_close(mdb);
 
-	exit(0);
+	return 0;
 }
 
