@@ -5,15 +5,15 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- *   
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "mdbtools.h"
@@ -23,32 +23,25 @@ main(int argc, char **argv)
 {
 
 #if 0
-	int i;
-	MdbCatalogEntry entry;
+	MdbHandle *mdb;
+	MdbTableDef *table;
 
-	if (argc<2) {
+	if (argc<3) {
 		fprintf(stderr,"Usage: %s <file> <table>\n",argv[0]);
 		exit(1);
 	}
 	
-	mdb_init();
-	mdb = mdb_open(argv[1]);
+	mdb = mdb_open(argv[1], MDB_NOFLAGS);
 
-	mdb_read_pg(mdb, MDB_CATALOG_PG);
-	rows = mdb_catalog_rows(mdb);
-
-	for (i=0;i<rows;i++) {
-  		if (mdb_read_catalog_entry(mdb, i, &entry)) {
-			if (!strcmp(entry.object_name,argv[2])) {
-				mdb_kkd_dump(&entry);
-  			}
-		}
+	table = mdb_read_table_by_name(mdb, argv[2], MDB_TABLE);
+	if (!table) {
+		fprintf(stderr,"No table named %s found.\n", argv[2]);
+		exit(1);
 	}
+	/* FIXME: table->entry->*kkd* are not initialized */
+	mdb_kkd_dump(table->entry);
 
-	mdb_free_handle(mdb);
-	mdb_exit();
+	mdb_close(mdb);
 #endif
-
-	exit(0);
+	return 0;
 }
-
