@@ -1,21 +1,19 @@
 /* MDB Tools - A library for reading MS Access database file
  * Copyright (C) 2000 Brian Bruns
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "mdbtools.h"
@@ -28,34 +26,22 @@ void print_table(MdbTableDef *table);
 
 main(int argc, char **argv)
 {
-int rows;
-int i;
-unsigned char buf[2048];
-MdbHandle *mdb;
-MdbCatalogEntry *entry;
-MdbTableDef *table;
+	MdbHandle *mdb;
+	MdbTableDef *table;
 
-	mdb_init();
-
-	if (!(mdb = mdb_open(MDB_FILE))) {
+	if (!(mdb = mdb_open(MDB_FILE, MDB_NOFLAGS))) {
 		exit(1);
 	}
 	
-	mdb_read_catalog(mdb, MDB_TABLE);
+	table = mdb_read_table_by_name(mdb, TABLE_NAME, MDB_TABLE);
 
-	for (i=0;i<mdb->num_catalog;i++) {
-		entry = g_ptr_array_index(mdb->catalog,i);
-		if (entry->object_type == MDB_TABLE &&
-			!strcmp(entry->object_name,TABLE_NAME)) {
-				table = mdb_read_table(entry);
-				print_table(table);
-		}
+	if (table) {
+		print_table(table);
+		mdb_free_tabledef(table);
 	}
 
-	mdb_free_handle(mdb);
-	mdb_exit();
-
-	exit(0);
+	mdb_close(mdb);
+	return 0;
 }
 
 void print_table(MdbTableDef *table)
@@ -76,9 +62,9 @@ MdbSarg sarg;
 	mdb_rewind_table(table);
 			
 	for (j=0;j<table->num_cols;j++) {
-		bound_values[j] = (char *) malloc(MDB_BIND_SIZE);
+		bound_values[j] = (char *) g_malloc(MDB_BIND_SIZE);
 		bound_values[j][0] = '\0';
-		mdb_bind_column(table, j+1, bound_values[j]);
+		mdb_bind_column(table, j+1, bound_values[j], NULL);
 	}
 
 	/* print header */
@@ -94,7 +80,6 @@ MdbSarg sarg;
 	while(mdb_fetch_row(table)) {
 		fprintf(stdout,"%s",bound_values[0]);
   		for (j=1;j<table->num_cols;j++) {
-			col=g_ptr_array_index(table->columns,j);
 			fprintf(stdout,"%s%s",DELIMETER,bound_values[j]);
 		}
 		fprintf(stdout,"\n");
@@ -102,6 +87,6 @@ MdbSarg sarg;
 
 	/* clean up */
 	for (j=0;j<table->num_cols;j++) {
-		free(bound_values[j]);
+		g_free(bound_values[j]);
 	}
 }
